# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer
from .validators.elasticbeanstalk import WebServer  # noqa: F401
from .validators.elasticbeanstalk import WebServerType  # noqa: F401
from .validators.elasticbeanstalk import Worker  # noqa: F401
from .validators.elasticbeanstalk import WorkerType  # noqa: F401
from .validators.elasticbeanstalk import validate_tier_name, validate_tier_type


class MaxAgeRule(AWSProperty):
    """
    `MaxAgeRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-application-maxagerule.html>`__
    """

    props: PropsDictType = {
        "DeleteSourceFromS3": (boolean, False),
        "Enabled": (boolean, False),
        "MaxAgeInDays": (integer, False),
    }


class MaxCountRule(AWSProperty):
    """
    `MaxCountRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-application-maxcountrule.html>`__
    """

    props: PropsDictType = {
        "DeleteSourceFromS3": (boolean, False),
        "Enabled": (boolean, False),
        "MaxCount": (integer, False),
    }


class ApplicationVersionLifecycleConfig(AWSProperty):
    """
    `ApplicationVersionLifecycleConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-application-applicationversionlifecycleconfig.html>`__
    """

    props: PropsDictType = {
        "MaxAgeRule": (MaxAgeRule, False),
        "MaxCountRule": (MaxCountRule, False),
    }


class ApplicationResourceLifecycleConfig(AWSProperty):
    """
    `ApplicationResourceLifecycleConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-application-applicationresourcelifecycleconfig.html>`__
    """

    props: PropsDictType = {
        "ServiceRole": (str, False),
        "VersionLifecycleConfig": (ApplicationVersionLifecycleConfig, False),
    }


class Application(AWSObject):
    """
    `Application <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticbeanstalk-application.html>`__
    """

    resource_type = "AWS::ElasticBeanstalk::Application"

    props: PropsDictType = {
        "ApplicationName": (str, False),
        "Description": (str, False),
        "ResourceLifecycleConfig": (ApplicationResourceLifecycleConfig, False),
    }


class SourceBundle(AWSProperty):
    """
    `SourceBundle <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-applicationversion-sourcebundle.html>`__
    """

    props: PropsDictType = {
        "S3Bucket": (str, True),
        "S3Key": (str, True),
    }


class ApplicationVersion(AWSObject):
    """
    `ApplicationVersion <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticbeanstalk-applicationversion.html>`__
    """

    resource_type = "AWS::ElasticBeanstalk::ApplicationVersion"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "Description": (str, False),
        "SourceBundle": (SourceBundle, True),
    }


class OptionSetting(AWSProperty):
    """
    `OptionSetting <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-environment-optionsetting.html>`__
    """

    props: PropsDictType = {
        "Namespace": (str, True),
        "OptionName": (str, True),
        "ResourceName": (str, False),
        "Value": (str, False),
    }


class SourceConfiguration(AWSProperty):
    """
    `SourceConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-configurationtemplate-sourceconfiguration.html>`__
    """

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "TemplateName": (str, True),
    }


class ConfigurationTemplate(AWSObject):
    """
    `ConfigurationTemplate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticbeanstalk-configurationtemplate.html>`__
    """

    resource_type = "AWS::ElasticBeanstalk::ConfigurationTemplate"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "Description": (str, False),
        "EnvironmentId": (str, False),
        "OptionSettings": ([OptionSetting], False),
        "PlatformArn": (str, False),
        "SolutionStackName": (str, False),
        "SourceConfiguration": (SourceConfiguration, False),
    }


class Tier(AWSProperty):
    """
    `Tier <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticbeanstalk-environment-tier.html>`__
    """

    props: PropsDictType = {
        "Name": (validate_tier_name, False),
        "Type": (validate_tier_type, False),
        "Version": (str, False),
    }


class Environment(AWSObject):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticbeanstalk-environment.html>`__
    """

    resource_type = "AWS::ElasticBeanstalk::Environment"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "CNAMEPrefix": (str, False),
        "Description": (str, False),
        "EnvironmentName": (str, False),
        "OperationsRole": (str, False),
        "OptionSettings": ([OptionSetting], False),
        "PlatformArn": (str, False),
        "SolutionStackName": (str, False),
        "Tags": (Tags, False),
        "TemplateName": (str, False),
        "Tier": (Tier, False),
        "VersionLabel": (str, False),
    }



import unittest

from troposphere.scheduler import EcsParameters, FlexibleTimeWindow


class TestSchedule(unittest.TestCase):
    def test_flexible_time_window_mode(self):
        for invalid_value in ("", "not-valid"):
            with self.assertRaises(ValueError):
                FlexibleTimeWindow(Mode=invalid_value).to_dict()

        for valid_value in ("OFF", "FLEXIBLE"):
            FlexibleTimeWindow(Mode=valid_value).to_dict()

    def test_ecsparameters_tags(self):
        with self.assertRaises(ValueError):
            EcsParameters(
                TaskDefinitionArn="some-arn",
                Tags={"tag1": "value1"},
            ).to_dict()

        EcsParameters(TaskDefinitionArn="some-arn").to_dict()


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def canary_runtime_version(runtime_version):
    """
    Property: Canary.RuntimeVersion
    """

    valid_runtime_versions = [
        "syn-nodejs-puppeteer-4.0",
        "syn-nodejs-puppeteer-5.0",
        "syn-nodejs-puppeteer-5.1",
        "syn-nodejs-puppeteer-6.0",
        "syn-nodejs-puppeteer-6.1",
    ]
    if runtime_version not in valid_runtime_versions:
        raise ValueError(
            'RuntimeVersion must be one of: "%s"' % (", ".join(valid_runtime_versions))
        )
    return runtime_version



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer


class ConnectAttachmentOptions(AWSProperty):
    """
    `ConnectAttachmentOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-connectattachment-connectattachmentoptions.html>`__
    """

    props: PropsDictType = {
        "Protocol": (str, False),
    }


class ProposedSegmentChange(AWSProperty):
    """
    `ProposedSegmentChange <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-vpcattachment-proposedsegmentchange.html>`__
    """

    props: PropsDictType = {
        "AttachmentPolicyRuleNumber": (integer, False),
        "SegmentName": (str, False),
        "Tags": (Tags, False),
    }


class ConnectAttachment(AWSObject):
    """
    `ConnectAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-connectattachment.html>`__
    """

    resource_type = "AWS::NetworkManager::ConnectAttachment"

    props: PropsDictType = {
        "CoreNetworkId": (str, True),
        "EdgeLocation": (str, True),
        "Options": (ConnectAttachmentOptions, True),
        "ProposedSegmentChange": (ProposedSegmentChange, False),
        "Tags": (Tags, False),
        "TransportAttachmentId": (str, True),
    }


class BgpOptions(AWSProperty):
    """
    `BgpOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-connectpeer-bgpoptions.html>`__
    """

    props: PropsDictType = {
        "PeerAsn": (double, False),
    }


class ConnectPeer(AWSObject):
    """
    `ConnectPeer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-connectpeer.html>`__
    """

    resource_type = "AWS::NetworkManager::ConnectPeer"

    props: PropsDictType = {
        "BgpOptions": (BgpOptions, False),
        "ConnectAttachmentId": (str, True),
        "CoreNetworkAddress": (str, False),
        "InsideCidrBlocks": ([str], False),
        "PeerAddress": (str, True),
        "SubnetArn": (str, False),
        "Tags": (Tags, False),
    }


class CoreNetwork(AWSObject):
    """
    `CoreNetwork <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-corenetwork.html>`__
    """

    resource_type = "AWS::NetworkManager::CoreNetwork"

    props: PropsDictType = {
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "PolicyDocument": (dict, False),
        "Tags": (Tags, False),
    }


class CustomerGatewayAssociation(AWSObject):
    """
    `CustomerGatewayAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-customergatewayassociation.html>`__
    """

    resource_type = "AWS::NetworkManager::CustomerGatewayAssociation"

    props: PropsDictType = {
        "CustomerGatewayArn": (str, True),
        "DeviceId": (str, True),
        "GlobalNetworkId": (str, True),
        "LinkId": (str, False),
    }


class AWSLocation(AWSProperty):
    """
    `AWSLocation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-device-awslocation.html>`__
    """

    props: PropsDictType = {
        "SubnetArn": (str, False),
        "Zone": (str, False),
    }


class Location(AWSProperty):
    """
    `Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-site-location.html>`__
    """

    props: PropsDictType = {
        "Address": (str, False),
        "Latitude": (str, False),
        "Longitude": (str, False),
    }


class Device(AWSObject):
    """
    `Device <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-device.html>`__
    """

    resource_type = "AWS::NetworkManager::Device"

    props: PropsDictType = {
        "AWSLocation": (AWSLocation, False),
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "Location": (Location, False),
        "Model": (str, False),
        "SerialNumber": (str, False),
        "SiteId": (str, False),
        "Tags": (Tags, False),
        "Type": (str, False),
        "Vendor": (str, False),
    }


class GlobalNetwork(AWSObject):
    """
    `GlobalNetwork <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-globalnetwork.html>`__
    """

    resource_type = "AWS::NetworkManager::GlobalNetwork"

    props: PropsDictType = {
        "CreatedAt": (str, False),
        "Description": (str, False),
        "State": (str, False),
        "Tags": (Tags, False),
    }


class Bandwidth(AWSProperty):
    """
    `Bandwidth <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-link-bandwidth.html>`__
    """

    props: PropsDictType = {
        "DownloadSpeed": (integer, False),
        "UploadSpeed": (integer, False),
    }


class Link(AWSObject):
    """
    `Link <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-link.html>`__
    """

    resource_type = "AWS::NetworkManager::Link"

    props: PropsDictType = {
        "Bandwidth": (Bandwidth, True),
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "Provider": (str, False),
        "SiteId": (str, True),
        "Tags": (Tags, False),
        "Type": (str, False),
    }


class LinkAssociation(AWSObject):
    """
    `LinkAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-linkassociation.html>`__
    """

    resource_type = "AWS::NetworkManager::LinkAssociation"

    props: PropsDictType = {
        "DeviceId": (str, True),
        "GlobalNetworkId": (str, True),
        "LinkId": (str, True),
    }


class Site(AWSObject):
    """
    `Site <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-site.html>`__
    """

    resource_type = "AWS::NetworkManager::Site"

    props: PropsDictType = {
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "Location": (Location, False),
        "Tags": (Tags, False),
    }


class SiteToSiteVpnAttachment(AWSObject):
    """
    `SiteToSiteVpnAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-sitetositevpnattachment.html>`__
    """

    resource_type = "AWS::NetworkManager::SiteToSiteVpnAttachment"

    props: PropsDictType = {
        "CoreNetworkId": (str, True),
        "ProposedSegmentChange": (ProposedSegmentChange, False),
        "Tags": (Tags, False),
        "VpnConnectionArn": (str, True),
    }


class TransitGatewayPeering(AWSObject):
    """
    `TransitGatewayPeering <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-transitgatewaypeering.html>`__
    """

    resource_type = "AWS::NetworkManager::TransitGatewayPeering"

    props: PropsDictType = {
        "CoreNetworkId": (str, True),
        "Tags": (Tags, False),
        "TransitGatewayArn": (str, True),
    }


class TransitGatewayRegistration(AWSObject):
    """
    `TransitGatewayRegistration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-transitgatewayregistration.html>`__
    """

    resource_type = "AWS::NetworkManager::TransitGatewayRegistration"

    props: PropsDictType = {
        "GlobalNetworkId": (str, True),
        "TransitGatewayArn": (str, True),
    }


class TransitGatewayRouteTableAttachment(AWSObject):
    """
    `TransitGatewayRouteTableAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-transitgatewayroutetableattachment.html>`__
    """

    resource_type = "AWS::NetworkManager::TransitGatewayRouteTableAttachment"

    props: PropsDictType = {
        "PeeringId": (str, True),
        "ProposedSegmentChange": (ProposedSegmentChange, False),
        "Tags": (Tags, False),
        "TransitGatewayRouteTableArn": (str, True),
    }


class VpcOptions(AWSProperty):
    """
    `VpcOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-vpcattachment-vpcoptions.html>`__
    """

    props: PropsDictType = {
        "ApplianceModeSupport": (boolean, False),
        "Ipv6Support": (boolean, False),
    }


class VpcAttachment(AWSObject):
    """
    `VpcAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-networkmanager-vpcattachment.html>`__
    """

    resource_type = "AWS::NetworkManager::VpcAttachment"

    props: PropsDictType = {
        "CoreNetworkId": (str, True),
        "Options": (VpcOptions, False),
        "ProposedSegmentChange": (ProposedSegmentChange, False),
        "SubnetArns": ([str], True),
        "Tags": (Tags, False),
        "VpcArn": (str, True),
    }


class ConnectPeerBgpConfiguration(AWSProperty):
    """
    `ConnectPeerBgpConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-connectpeer-connectpeerbgpconfiguration.html>`__
    """

    props: PropsDictType = {
        "CoreNetworkAddress": (str, False),
        "CoreNetworkAsn": (double, False),
        "PeerAddress": (str, False),
        "PeerAsn": (double, False),
    }


class ConnectPeerConfiguration(AWSProperty):
    """
    `ConnectPeerConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-connectpeer-connectpeerconfiguration.html>`__
    """

    props: PropsDictType = {
        "BgpConfigurations": ([ConnectPeerBgpConfiguration], False),
        "CoreNetworkAddress": (str, False),
        "InsideCidrBlocks": ([str], False),
        "PeerAddress": (str, False),
        "Protocol": (str, False),
    }


class CoreNetworkEdge(AWSProperty):
    """
    `CoreNetworkEdge <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-corenetwork-corenetworkedge.html>`__
    """

    props: PropsDictType = {
        "Asn": (double, False),
        "EdgeLocation": (str, False),
        "InsideCidrBlocks": ([str], False),
    }


class CoreNetworkSegment(AWSProperty):
    """
    `CoreNetworkSegment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-networkmanager-corenetwork-corenetworksegment.html>`__
    """

    props: PropsDictType = {
        "EdgeLocations": ([str], False),
        "Name": (str, False),
        "SharedSegments": ([str], False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class Endpoint(AWSProperty):
    """
    `Endpoint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-cluster-endpoint.html>`__
    """

    props: PropsDictType = {
        "Address": (str, False),
        "Port": (str, False),
    }


class LoggingProperties(AWSProperty):
    """
    `LoggingProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-cluster-loggingproperties.html>`__
    """

    props: PropsDictType = {
        "BucketName": (str, False),
        "S3KeyPrefix": (str, False),
    }


class Cluster(AWSObject):
    """
    `Cluster <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html>`__
    """

    resource_type = "AWS::Redshift::Cluster"

    props: PropsDictType = {
        "AllowVersionUpgrade": (boolean, False),
        "AquaConfigurationStatus": (str, False),
        "AutomatedSnapshotRetentionPeriod": (integer, False),
        "AvailabilityZone": (str, False),
        "AvailabilityZoneRelocation": (boolean, False),
        "AvailabilityZoneRelocationStatus": (str, False),
        "Classic": (boolean, False),
        "ClusterIdentifier": (str, False),
        "ClusterParameterGroupName": (str, False),
        "ClusterSecurityGroups": ([str], False),
        "ClusterSubnetGroupName": (str, False),
        "ClusterType": (str, True),
        "ClusterVersion": (str, False),
        "DBName": (str, True),
        "DeferMaintenance": (boolean, False),
        "DeferMaintenanceDuration": (integer, False),
        "DeferMaintenanceEndTime": (str, False),
        "DeferMaintenanceStartTime": (str, False),
        "DestinationRegion": (str, False),
        "ElasticIp": (str, False),
        "Encrypted": (boolean, False),
        "Endpoint": (Endpoint, False),
        "EnhancedVpcRouting": (boolean, False),
        "HsmClientCertificateIdentifier": (str, False),
        "HsmConfigurationIdentifier": (str, False),
        "IamRoles": ([str], False),
        "KmsKeyId": (str, False),
        "LoggingProperties": (LoggingProperties, False),
        "MaintenanceTrackName": (str, False),
        "ManageMasterPassword": (boolean, False),
        "ManualSnapshotRetentionPeriod": (integer, False),
        "MasterPasswordSecretKmsKeyId": (str, False),
        "MasterUserPassword": (str, False),
        "MasterUsername": (str, True),
        "MultiAZ": (boolean, False),
        "NamespaceResourcePolicy": (dict, False),
        "NodeType": (str, True),
        "NumberOfNodes": (integer, False),
        "OwnerAccount": (str, False),
        "Port": (integer, False),
        "PreferredMaintenanceWindow": (str, False),
        "PubliclyAccessible": (boolean, False),
        "ResourceAction": (str, False),
        "RevisionTarget": (str, False),
        "RotateEncryptionKey": (boolean, False),
        "SnapshotClusterIdentifier": (str, False),
        "SnapshotCopyGrantName": (str, False),
        "SnapshotCopyManual": (boolean, False),
        "SnapshotCopyRetentionPeriod": (integer, False),
        "SnapshotIdentifier": (str, False),
        "Tags": (Tags, False),
        "VpcSecurityGroupIds": ([str], False),
    }


class AmazonRedshiftParameter(AWSProperty):
    """
    `AmazonRedshiftParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-clusterparametergroup-parameter.html>`__
    """

    props: PropsDictType = {
        "ParameterName": (str, True),
        "ParameterValue": (str, True),
    }


class ClusterParameterGroup(AWSObject):
    """
    `ClusterParameterGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html>`__
    """

    resource_type = "AWS::Redshift::ClusterParameterGroup"

    props: PropsDictType = {
        "Description": (str, True),
        "ParameterGroupFamily": (str, True),
        "ParameterGroupName": (str, False),
        "Parameters": ([AmazonRedshiftParameter], False),
        "Tags": (Tags, False),
    }


class ClusterSecurityGroup(AWSObject):
    """
    `ClusterSecurityGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroup.html>`__
    """

    resource_type = "AWS::Redshift::ClusterSecurityGroup"

    props: PropsDictType = {
        "Description": (str, True),
        "Tags": (Tags, False),
    }


class ClusterSecurityGroupIngress(AWSObject):
    """
    `ClusterSecurityGroupIngress <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html>`__
    """

    resource_type = "AWS::Redshift::ClusterSecurityGroupIngress"

    props: PropsDictType = {
        "CIDRIP": (str, False),
        "ClusterSecurityGroupName": (str, True),
        "EC2SecurityGroupName": (str, False),
        "EC2SecurityGroupOwnerId": (str, False),
    }


class ClusterSubnetGroup(AWSObject):
    """
    `ClusterSubnetGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html>`__
    """

    resource_type = "AWS::Redshift::ClusterSubnetGroup"

    props: PropsDictType = {
        "Description": (str, True),
        "SubnetIds": ([str], True),
        "Tags": (Tags, False),
    }


class EndpointAccess(AWSObject):
    """
    `EndpointAccess <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-endpointaccess.html>`__
    """

    resource_type = "AWS::Redshift::EndpointAccess"

    props: PropsDictType = {
        "ClusterIdentifier": (str, True),
        "EndpointName": (str, True),
        "ResourceOwner": (str, False),
        "SubnetGroupName": (str, True),
        "VpcSecurityGroupIds": ([str], True),
    }


class EndpointAuthorization(AWSObject):
    """
    `EndpointAuthorization <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-endpointauthorization.html>`__
    """

    resource_type = "AWS::Redshift::EndpointAuthorization"

    props: PropsDictType = {
        "Account": (str, True),
        "ClusterIdentifier": (str, True),
        "Force": (boolean, False),
        "VpcIds": ([str], False),
    }


class EventSubscription(AWSObject):
    """
    `EventSubscription <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-eventsubscription.html>`__
    """

    resource_type = "AWS::Redshift::EventSubscription"

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "EventCategories": ([str], False),
        "Severity": (str, False),
        "SnsTopicArn": (str, False),
        "SourceIds": ([str], False),
        "SourceType": (str, False),
        "SubscriptionName": (str, True),
        "Tags": (Tags, False),
    }


class PauseClusterMessage(AWSProperty):
    """
    `PauseClusterMessage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-scheduledaction-pauseclustermessage.html>`__
    """

    props: PropsDictType = {
        "ClusterIdentifier": (str, True),
    }


class ResizeClusterMessage(AWSProperty):
    """
    `ResizeClusterMessage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-scheduledaction-resizeclustermessage.html>`__
    """

    props: PropsDictType = {
        "Classic": (boolean, False),
        "ClusterIdentifier": (str, True),
        "ClusterType": (str, False),
        "NodeType": (str, False),
        "NumberOfNodes": (integer, False),
    }


class ResumeClusterMessage(AWSProperty):
    """
    `ResumeClusterMessage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-scheduledaction-resumeclustermessage.html>`__
    """

    props: PropsDictType = {
        "ClusterIdentifier": (str, True),
    }


class ScheduledActionType(AWSProperty):
    """
    `ScheduledActionType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-scheduledaction-scheduledactiontype.html>`__
    """

    props: PropsDictType = {
        "PauseCluster": (PauseClusterMessage, False),
        "ResizeCluster": (ResizeClusterMessage, False),
        "ResumeCluster": (ResumeClusterMessage, False),
    }


class ScheduledAction(AWSObject):
    """
    `ScheduledAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-scheduledaction.html>`__
    """

    resource_type = "AWS::Redshift::ScheduledAction"

    props: PropsDictType = {
        "Enable": (boolean, False),
        "EndTime": (str, False),
        "IamRole": (str, False),
        "Schedule": (str, False),
        "ScheduledActionDescription": (str, False),
        "ScheduledActionName": (str, True),
        "StartTime": (str, False),
        "TargetAction": (ScheduledActionType, False),
    }


class NetworkInterface(AWSProperty):
    """
    `NetworkInterface <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-endpointaccess-networkinterface.html>`__
    """

    props: PropsDictType = {
        "AvailabilityZone": (str, False),
        "NetworkInterfaceId": (str, False),
        "PrivateIpAddress": (str, False),
        "SubnetId": (str, False),
    }


class VpcEndpoint(AWSProperty):
    """
    `VpcEndpoint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-endpointaccess-vpcendpoint.html>`__
    """

    props: PropsDictType = {
        "NetworkInterfaces": ([NetworkInterface], False),
        "VpcEndpointId": (str, False),
        "VpcId": (str, False),
    }


class VpcSecurityGroup(AWSProperty):
    """
    `VpcSecurityGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-endpointaccess-vpcsecuritygroup.html>`__
    """

    props: PropsDictType = {
        "Status": (str, False),
        "VpcSecurityGroupId": (str, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer
from .validators.appstream import validate_tags_or_list


class S3Location(AWSProperty):
    """
    `S3Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-fleet-s3location.html>`__
    """

    props: PropsDictType = {
        "S3Bucket": (str, True),
        "S3Key": (str, True),
    }


class ScriptDetails(AWSProperty):
    """
    `ScriptDetails <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-appblock-scriptdetails.html>`__
    """

    props: PropsDictType = {
        "ExecutableParameters": (str, False),
        "ExecutablePath": (str, True),
        "ScriptS3Location": (S3Location, True),
        "TimeoutInSeconds": (integer, True),
    }


class AppBlock(AWSObject):
    """
    `AppBlock <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-appblock.html>`__
    """

    resource_type = "AWS::AppStream::AppBlock"

    props: PropsDictType = {
        "Description": (str, False),
        "DisplayName": (str, False),
        "Name": (str, True),
        "PackagingType": (str, False),
        "PostSetupScriptDetails": (ScriptDetails, False),
        "SetupScriptDetails": (ScriptDetails, False),
        "SourceS3Location": (S3Location, True),
        "Tags": (validate_tags_or_list, False),
    }


class AccessEndpoint(AWSProperty):
    """
    `AccessEndpoint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-stack-accessendpoint.html>`__
    """

    props: PropsDictType = {
        "EndpointType": (str, True),
        "VpceId": (str, True),
    }


class VpcConfig(AWSProperty):
    """
    `VpcConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-imagebuilder-vpcconfig.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupIds": ([str], False),
        "SubnetIds": ([str], False),
    }


class AppBlockBuilder(AWSObject):
    """
    `AppBlockBuilder <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-appblockbuilder.html>`__
    """

    resource_type = "AWS::AppStream::AppBlockBuilder"

    props: PropsDictType = {
        "AccessEndpoints": ([AccessEndpoint], False),
        "AppBlockArns": ([str], False),
        "Description": (str, False),
        "DisplayName": (str, False),
        "EnableDefaultInternetAccess": (boolean, False),
        "IamRoleArn": (str, False),
        "InstanceType": (str, True),
        "Name": (str, True),
        "Platform": (str, True),
        "Tags": (Tags, False),
        "VpcConfig": (VpcConfig, True),
    }


class Application(AWSObject):
    """
    `Application <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-application.html>`__
    """

    resource_type = "AWS::AppStream::Application"

    props: PropsDictType = {
        "AppBlockArn": (str, True),
        "AttributesToDelete": ([str], False),
        "Description": (str, False),
        "DisplayName": (str, False),
        "IconS3Location": (S3Location, True),
        "InstanceFamilies": ([str], True),
        "LaunchParameters": (str, False),
        "LaunchPath": (str, True),
        "Name": (str, True),
        "Platforms": ([str], True),
        "Tags": (validate_tags_or_list, False),
        "WorkingDirectory": (str, False),
    }


class ApplicationEntitlementAssociation(AWSObject):
    """
    `ApplicationEntitlementAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-applicationentitlementassociation.html>`__
    """

    resource_type = "AWS::AppStream::ApplicationEntitlementAssociation"

    props: PropsDictType = {
        "ApplicationIdentifier": (str, True),
        "EntitlementName": (str, True),
        "StackName": (str, True),
    }


class ApplicationFleetAssociation(AWSObject):
    """
    `ApplicationFleetAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-applicationfleetassociation.html>`__
    """

    resource_type = "AWS::AppStream::ApplicationFleetAssociation"

    props: PropsDictType = {
        "ApplicationArn": (str, True),
        "FleetName": (str, True),
    }


class CertificateBasedAuthProperties(AWSProperty):
    """
    `CertificateBasedAuthProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-directoryconfig-certificatebasedauthproperties.html>`__
    """

    props: PropsDictType = {
        "CertificateAuthorityArn": (str, False),
        "Status": (str, False),
    }


class ServiceAccountCredentials(AWSProperty):
    """
    `ServiceAccountCredentials <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-directoryconfig-serviceaccountcredentials.html>`__
    """

    props: PropsDictType = {
        "AccountName": (str, True),
        "AccountPassword": (str, True),
    }


class DirectoryConfig(AWSObject):
    """
    `DirectoryConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-directoryconfig.html>`__
    """

    resource_type = "AWS::AppStream::DirectoryConfig"

    props: PropsDictType = {
        "CertificateBasedAuthProperties": (CertificateBasedAuthProperties, False),
        "DirectoryName": (str, True),
        "OrganizationalUnitDistinguishedNames": ([str], True),
        "ServiceAccountCredentials": (ServiceAccountCredentials, True),
    }


class Attribute(AWSProperty):
    """
    `Attribute <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-entitlement-attribute.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "Value": (str, True),
    }


class Entitlement(AWSObject):
    """
    `Entitlement <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-entitlement.html>`__
    """

    resource_type = "AWS::AppStream::Entitlement"

    props: PropsDictType = {
        "AppVisibility": (str, True),
        "Attributes": ([Attribute], True),
        "Description": (str, False),
        "Name": (str, True),
        "StackName": (str, True),
    }


class ComputeCapacity(AWSProperty):
    """
    `ComputeCapacity <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-fleet-computecapacity.html>`__
    """

    props: PropsDictType = {
        "DesiredInstances": (integer, False),
        "DesiredSessions": (integer, False),
    }


class DomainJoinInfo(AWSProperty):
    """
    `DomainJoinInfo <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-imagebuilder-domainjoininfo.html>`__
    """

    props: PropsDictType = {
        "DirectoryName": (str, False),
        "OrganizationalUnitDistinguishedName": (str, False),
    }


class Fleet(AWSObject):
    """
    `Fleet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-fleet.html>`__
    """

    resource_type = "AWS::AppStream::Fleet"

    props: PropsDictType = {
        "ComputeCapacity": (ComputeCapacity, False),
        "Description": (str, False),
        "DisconnectTimeoutInSeconds": (integer, False),
        "DisplayName": (str, False),
        "DomainJoinInfo": (DomainJoinInfo, False),
        "EnableDefaultInternetAccess": (boolean, False),
        "FleetType": (str, False),
        "IamRoleArn": (str, False),
        "IdleDisconnectTimeoutInSeconds": (integer, False),
        "ImageArn": (str, False),
        "ImageName": (str, False),
        "InstanceType": (str, True),
        "MaxConcurrentSessions": (integer, False),
        "MaxSessionsPerInstance": (integer, False),
        "MaxUserDurationInSeconds": (integer, False),
        "Name": (str, True),
        "Platform": (str, False),
        "SessionScriptS3Location": (S3Location, False),
        "StreamView": (str, False),
        "Tags": (validate_tags_or_list, False),
        "UsbDeviceFilterStrings": ([str], False),
        "VpcConfig": (VpcConfig, False),
    }


class ImageBuilder(AWSObject):
    """
    `ImageBuilder <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-imagebuilder.html>`__
    """

    resource_type = "AWS::AppStream::ImageBuilder"

    props: PropsDictType = {
        "AccessEndpoints": ([AccessEndpoint], False),
        "AppstreamAgentVersion": (str, False),
        "Description": (str, False),
        "DisplayName": (str, False),
        "DomainJoinInfo": (DomainJoinInfo, False),
        "EnableDefaultInternetAccess": (boolean, False),
        "IamRoleArn": (str, False),
        "ImageArn": (str, False),
        "ImageName": (str, False),
        "InstanceType": (str, True),
        "Name": (str, True),
        "Tags": (validate_tags_or_list, False),
        "VpcConfig": (VpcConfig, False),
    }


class ApplicationSettings(AWSProperty):
    """
    `ApplicationSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-stack-applicationsettings.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, True),
        "SettingsGroup": (str, False),
    }


class StorageConnector(AWSProperty):
    """
    `StorageConnector <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-stack-storageconnector.html>`__
    """

    props: PropsDictType = {
        "ConnectorType": (str, True),
        "Domains": ([str], False),
        "ResourceIdentifier": (str, False),
    }


class StreamingExperienceSettings(AWSProperty):
    """
    `StreamingExperienceSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-stack-streamingexperiencesettings.html>`__
    """

    props: PropsDictType = {
        "PreferredProtocol": (str, False),
    }


class UserSetting(AWSProperty):
    """
    `UserSetting <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-appstream-stack-usersetting.html>`__
    """

    props: PropsDictType = {
        "Action": (str, True),
        "MaximumLength": (integer, False),
        "Permission": (str, True),
    }


class Stack(AWSObject):
    """
    `Stack <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-stack.html>`__
    """

    resource_type = "AWS::AppStream::Stack"

    props: PropsDictType = {
        "AccessEndpoints": ([AccessEndpoint], False),
        "ApplicationSettings": (ApplicationSettings, False),
        "AttributesToDelete": ([str], False),
        "DeleteStorageConnectors": (boolean, False),
        "Description": (str, False),
        "DisplayName": (str, False),
        "EmbedHostDomains": ([str], False),
        "FeedbackURL": (str, False),
        "Name": (str, False),
        "RedirectURL": (str, False),
        "StorageConnectors": ([StorageConnector], False),
        "StreamingExperienceSettings": (StreamingExperienceSettings, False),
        "Tags": (validate_tags_or_list, False),
        "UserSettings": ([UserSetting], False),
    }


class StackFleetAssociation(AWSObject):
    """
    `StackFleetAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-stackfleetassociation.html>`__
    """

    resource_type = "AWS::AppStream::StackFleetAssociation"

    props: PropsDictType = {
        "FleetName": (str, True),
        "StackName": (str, True),
    }


class StackUserAssociation(AWSObject):
    """
    `StackUserAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-stackuserassociation.html>`__
    """

    resource_type = "AWS::AppStream::StackUserAssociation"

    props: PropsDictType = {
        "AuthenticationType": (str, True),
        "SendEmailNotification": (boolean, False),
        "StackName": (str, True),
        "UserName": (str, True),
    }


class User(AWSObject):
    """
    `User <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-user.html>`__
    """

    resource_type = "AWS::AppStream::User"

    props: PropsDictType = {
        "AuthenticationType": (str, True),
        "FirstName": (str, False),
        "LastName": (str, False),
        "MessageAction": (str, False),
        "UserName": (str, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import double


class DefinitionDocument(AWSProperty):
    """
    `DefinitionDocument <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iotthingsgraph-flowtemplate-definitiondocument.html>`__
    """

    props: PropsDictType = {
        "Language": (str, True),
        "Text": (str, True),
    }


class FlowTemplate(AWSObject):
    """
    `FlowTemplate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotthingsgraph-flowtemplate.html>`__
    """

    resource_type = "AWS::IoTThingsGraph::FlowTemplate"

    props: PropsDictType = {
        "CompatibleNamespaceVersion": (double, False),
        "Definition": (DefinitionDocument, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean


class ManifestOverridesPayload(AWSProperty):
    """
    `ManifestOverridesPayload <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-panorama-applicationinstance-manifestoverridespayload.html>`__
    """

    props: PropsDictType = {
        "PayloadData": (str, False),
    }


class ManifestPayload(AWSProperty):
    """
    `ManifestPayload <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-panorama-applicationinstance-manifestpayload.html>`__
    """

    props: PropsDictType = {
        "PayloadData": (str, False),
    }


class ApplicationInstance(AWSObject):
    """
    `ApplicationInstance <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-panorama-applicationinstance.html>`__
    """

    resource_type = "AWS::Panorama::ApplicationInstance"

    props: PropsDictType = {
        "ApplicationInstanceIdToReplace": (str, False),
        "DefaultRuntimeContextDevice": (str, True),
        "Description": (str, False),
        "ManifestOverridesPayload": (ManifestOverridesPayload, False),
        "ManifestPayload": (ManifestPayload, True),
        "Name": (str, False),
        "RuntimeRoleArn": (str, False),
        "Tags": (Tags, False),
    }


class StorageLocation(AWSProperty):
    """
    `StorageLocation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-panorama-package-storagelocation.html>`__
    """

    props: PropsDictType = {
        "BinaryPrefixLocation": (str, False),
        "Bucket": (str, False),
        "GeneratedPrefixLocation": (str, False),
        "ManifestPrefixLocation": (str, False),
        "RepoPrefixLocation": (str, False),
    }


class Package(AWSObject):
    """
    `Package <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-panorama-package.html>`__
    """

    resource_type = "AWS::Panorama::Package"

    props: PropsDictType = {
        "PackageName": (str, True),
        "StorageLocation": (StorageLocation, False),
        "Tags": (Tags, False),
    }


class PackageVersion(AWSObject):
    """
    `PackageVersion <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-panorama-packageversion.html>`__
    """

    resource_type = "AWS::Panorama::PackageVersion"

    props: PropsDictType = {
        "MarkLatest": (boolean, False),
        "OwnerAccount": (str, False),
        "PackageId": (str, True),
        "PackageVersion": (str, True),
        "PatchVersion": (str, True),
        "UpdatedLatestPatchVersion": (str, False),
    }



"""
Batch.

Create the AWS Batch Compute environment and roles.
"""

from troposphere import GetAtt, Output, Parameter, Ref, Tags, Template
from troposphere.batch import (
    ComputeEnvironment,
    ComputeEnvironmentOrder,
    ComputeResources,
    JobQueue,
    LaunchTemplateSpecification,
)
from troposphere.ec2 import SecurityGroup
from troposphere.iam import InstanceProfile, Role

t = Template("AWS Batch")
t.set_version()

PrivateSubnetA = t.add_parameter(Parameter("PrivateSubnetA", Type="String"))

PrivateSubnetB = t.add_parameter(Parameter("PrivateSubnetB", Type="String"))

Vpc = t.add_parameter(
    Parameter(
        "Vpc", ConstraintDescription="Must be a valid VPC ID.", Type="AWS::EC2::VPC::Id"
    )
)

BatchServiceRole = t.add_resource(
    Role(
        "BatchServiceRole",
        Path="/",
        Policies=[],
        ManagedPolicyArns=[
            "arn:aws:iam::aws:policy/service-role/AWSBatchServiceRole",
        ],
        AssumeRolePolicyDocument={
            "Statement": [
                {
                    "Action": ["sts:AssumeRole"],
                    "Effect": "Allow",
                    "Principal": {"Service": ["batch.amazonaws.com"]},
                }
            ]
        },
    )
)

BatchInstanceRole = t.add_resource(
    Role(
        "BatchInstanceRole",
        Path="/",
        Policies=[],
        ManagedPolicyArns=[
            "arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceforEC2Role",  # NOQA
        ],
        AssumeRolePolicyDocument={
            "Statement": [
                {
                    "Action": ["sts:AssumeRole"],
                    "Effect": "Allow",
                    "Principal": {"Service": ["ec2.amazonaws.com"]},
                }
            ]
        },
    )
)

BatchInstanceProfile = t.add_resource(
    InstanceProfile(
        "BatchInstanceProfile",
        Path="/",
        Roles=[Ref(BatchInstanceRole)],
    )
)

BatchSecurityGroup = t.add_resource(
    SecurityGroup(
        "BatchSecurityGroup",
        VpcId=Ref(Vpc),
        GroupDescription="Enable access to Batch instances",
        Tags=Tags(Name="batch-sg"),
    )
)

BatchComputeEnvironment = t.add_resource(
    ComputeEnvironment(
        "BatchComputeEnvironment",
        Type="MANAGED",
        ServiceRole=Ref(BatchServiceRole),
        ComputeResources=ComputeResources(
            "BatchComputeResources",
            Type="EC2",
            DesiredvCpus=0,
            MinvCpus=0,
            MaxvCpus=10,
            PlacementGroup="ExampleClusterGroup",
            InstanceTypes=["m4.large"],
            LaunchTemplate=LaunchTemplateSpecification(
                LaunchTemplateName="container-volume-encrypt"
            ),
            InstanceRole=Ref(BatchInstanceProfile),
            SecurityGroupIds=[GetAtt(BatchSecurityGroup, "GroupId")],
            Subnets=[Ref(PrivateSubnetA), Ref(PrivateSubnetB)],
            Tags=dict(Name="batch-compute-environment", Project="example"),
        ),
    )
)

ExampleJobQueue = t.add_resource(
    JobQueue(
        "ExampleJobQueue",
        ComputeEnvironmentOrder=[
            ComputeEnvironmentOrder(
                ComputeEnvironment=Ref(BatchComputeEnvironment), Order=1
            ),
        ],
        Priority=1,
        State="ENABLED",
        JobQueueName="example-job-queue",
    )
)

t.add_output(
    [
        Output("BatchComputeEnvironment", Value=Ref(BatchComputeEnvironment)),
        Output("BatchSecurityGroup", Value=Ref(BatchSecurityGroup)),
        Output("ExampleJobQueue", Value=Ref(ExampleJobQueue)),
    ]
)

print(t.to_json())



import unittest

from troposphere import Join
from troposphere.apigatewayv2 import Authorizer, IntegrationResponse, Model


class TestModel(unittest.TestCase):
    def test_schema(self):
        # Check with no schema
        model = Model(
            "schema",
            Name="model",
            ApiId="apiid",
        )
        model.validate()

        # Check valid json schema string
        model = Model(
            "schema",
            ApiId="apiid",
            Name="model",
            Schema='{"a": "b"}',
        )
        model.validate()

        # Check invalid json schema string
        model = Model(
            "schema",
            ApiId="apiid",
            Name="model",
            Schema='{"a: "b"}',
        )
        with self.assertRaises(ValueError):
            model.validate()

        # Check accepting dict and converting to string in validate
        d = {"c": "d"}
        model = Model("schema", ApiId="apiid", Name="model", Schema=d)
        model.validate()
        self.assertEqual(model.properties["Schema"], '{"c": "d"}')

        # Check invalid Schema type
        with self.assertRaises(TypeError):
            model = Model("schema", ApiId="apiid", Name="model", Schema=1)

        # Check Schema being an AWSHelperFn
        model = Model(
            "schema",
            ApiId="apiid",
            Name="model",
            Schema=Join(":", ['{"a', ': "b"}']),
        )
        model.validate()


class TestIntegrationResponse(unittest.TestCase):
    def test_response_type(self):
        integration_response = IntegrationResponse(
            "IntegrationResponse",
            IntegrationId="integrationid",
            ApiId="apiid",
            IntegrationResponseKey="/400/",
        )

        integration_response.validate()

        with self.assertRaises(ValueError):
            integration_response = IntegrationResponse(
                "GatewayResponse",
                IntegrationId="integrationid",
                ApiId="apiid",
                IntegrationResponseKey="/400/",
                ContentHandlingStrategy="CONVERT_TO_SOMETHING",
            )


class TestAuthorizer(unittest.TestCase):
    def test_response_type(self):
        authorizer = Authorizer(
            "Authorizer",
            ApiId="apiid",
            AuthorizerType="REQUEST",
            AuthorizerUri="arn:lambda:function",
        )

        authorizer.validate()

        with self.assertRaises(ValueError):
            authorizer = Authorizer(
                "Authorizer",
                ApiId="apiid",
                AuthorizerType="RESPONSE",
                AuthorizerUri="arn:lambda:function",
            )


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class AccessLogSubscription(AWSObject):
    """
    `AccessLogSubscription <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-accesslogsubscription.html>`__
    """

    resource_type = "AWS::VpcLattice::AccessLogSubscription"

    props: PropsDictType = {
        "DestinationArn": (str, True),
        "ResourceIdentifier": (str, False),
        "Tags": (Tags, False),
    }


class AuthPolicy(AWSObject):
    """
    `AuthPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-authpolicy.html>`__
    """

    resource_type = "AWS::VpcLattice::AuthPolicy"

    props: PropsDictType = {
        "Policy": (dict, True),
        "ResourceIdentifier": (str, True),
    }


class FixedResponse(AWSProperty):
    """
    `FixedResponse <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-fixedresponse.html>`__
    """

    props: PropsDictType = {
        "StatusCode": (integer, True),
    }


class WeightedTargetGroup(AWSProperty):
    """
    `WeightedTargetGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-weightedtargetgroup.html>`__
    """

    props: PropsDictType = {
        "TargetGroupIdentifier": (str, True),
        "Weight": (integer, False),
    }


class Forward(AWSProperty):
    """
    `Forward <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-forward.html>`__
    """

    props: PropsDictType = {
        "TargetGroups": ([WeightedTargetGroup], True),
    }


class DefaultAction(AWSProperty):
    """
    `DefaultAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-listener-defaultaction.html>`__
    """

    props: PropsDictType = {
        "FixedResponse": (FixedResponse, False),
        "Forward": (Forward, False),
    }


class Listener(AWSObject):
    """
    `Listener <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-listener.html>`__
    """

    resource_type = "AWS::VpcLattice::Listener"

    props: PropsDictType = {
        "DefaultAction": (DefaultAction, True),
        "Name": (str, False),
        "Port": (integer, False),
        "Protocol": (str, True),
        "ServiceIdentifier": (str, False),
        "Tags": (Tags, False),
    }


class ResourcePolicy(AWSObject):
    """
    `ResourcePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-resourcepolicy.html>`__
    """

    resource_type = "AWS::VpcLattice::ResourcePolicy"

    props: PropsDictType = {
        "Policy": (dict, True),
        "ResourceArn": (str, True),
    }


class Action(AWSProperty):
    """
    `Action <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-action.html>`__
    """

    props: PropsDictType = {
        "FixedResponse": (FixedResponse, False),
        "Forward": (Forward, False),
    }


class HeaderMatchType(AWSProperty):
    """
    `HeaderMatchType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-headermatchtype.html>`__
    """

    props: PropsDictType = {
        "Contains": (str, False),
        "Exact": (str, False),
        "Prefix": (str, False),
    }


class HeaderMatch(AWSProperty):
    """
    `HeaderMatch <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-headermatch.html>`__
    """

    props: PropsDictType = {
        "CaseSensitive": (boolean, False),
        "Match": (HeaderMatchType, True),
        "Name": (str, True),
    }


class PathMatchType(AWSProperty):
    """
    `PathMatchType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-pathmatchtype.html>`__
    """

    props: PropsDictType = {
        "Exact": (str, False),
        "Prefix": (str, False),
    }


class PathMatch(AWSProperty):
    """
    `PathMatch <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-pathmatch.html>`__
    """

    props: PropsDictType = {
        "CaseSensitive": (boolean, False),
        "Match": (PathMatchType, True),
    }


class HttpMatch(AWSProperty):
    """
    `HttpMatch <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-httpmatch.html>`__
    """

    props: PropsDictType = {
        "HeaderMatches": ([HeaderMatch], False),
        "Method": (str, False),
        "PathMatch": (PathMatch, False),
    }


class Match(AWSProperty):
    """
    `Match <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-rule-match.html>`__
    """

    props: PropsDictType = {
        "HttpMatch": (HttpMatch, True),
    }


class Rule(AWSObject):
    """
    `Rule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-rule.html>`__
    """

    resource_type = "AWS::VpcLattice::Rule"

    props: PropsDictType = {
        "Action": (Action, True),
        "ListenerIdentifier": (str, False),
        "Match": (Match, True),
        "Name": (str, False),
        "Priority": (integer, True),
        "ServiceIdentifier": (str, False),
        "Tags": (Tags, False),
    }


class DnsEntry(AWSProperty):
    """
    `DnsEntry <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-servicenetworkserviceassociation-dnsentry.html>`__
    """

    props: PropsDictType = {
        "DomainName": (str, False),
        "HostedZoneId": (str, False),
    }


class Service(AWSObject):
    """
    `Service <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-service.html>`__
    """

    resource_type = "AWS::VpcLattice::Service"

    props: PropsDictType = {
        "AuthType": (str, False),
        "CertificateArn": (str, False),
        "CustomDomainName": (str, False),
        "DnsEntry": (DnsEntry, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class ServiceNetwork(AWSObject):
    """
    `ServiceNetwork <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-servicenetwork.html>`__
    """

    resource_type = "AWS::VpcLattice::ServiceNetwork"

    props: PropsDictType = {
        "AuthType": (str, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class ServiceNetworkServiceAssociation(AWSObject):
    """
    `ServiceNetworkServiceAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-servicenetworkserviceassociation.html>`__
    """

    resource_type = "AWS::VpcLattice::ServiceNetworkServiceAssociation"

    props: PropsDictType = {
        "DnsEntry": (DnsEntry, False),
        "ServiceIdentifier": (str, False),
        "ServiceNetworkIdentifier": (str, False),
        "Tags": (Tags, False),
    }


class ServiceNetworkVpcAssociation(AWSObject):
    """
    `ServiceNetworkVpcAssociation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-servicenetworkvpcassociation.html>`__
    """

    resource_type = "AWS::VpcLattice::ServiceNetworkVpcAssociation"

    props: PropsDictType = {
        "SecurityGroupIds": ([str], False),
        "ServiceNetworkIdentifier": (str, False),
        "Tags": (Tags, False),
        "VpcIdentifier": (str, False),
    }


class Target(AWSProperty):
    """
    `Target <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-targetgroup-target.html>`__
    """

    props: PropsDictType = {
        "Id": (str, True),
        "Port": (integer, False),
    }


class Matcher(AWSProperty):
    """
    `Matcher <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-targetgroup-matcher.html>`__
    """

    props: PropsDictType = {
        "HttpCode": (str, True),
    }


class HealthCheckConfig(AWSProperty):
    """
    `HealthCheckConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-targetgroup-healthcheckconfig.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "HealthCheckIntervalSeconds": (integer, False),
        "HealthCheckTimeoutSeconds": (integer, False),
        "HealthyThresholdCount": (integer, False),
        "Matcher": (Matcher, False),
        "Path": (str, False),
        "Port": (integer, False),
        "Protocol": (str, False),
        "ProtocolVersion": (str, False),
        "UnhealthyThresholdCount": (integer, False),
    }


class TargetGroupConfig(AWSProperty):
    """
    `TargetGroupConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-vpclattice-targetgroup-targetgroupconfig.html>`__
    """

    props: PropsDictType = {
        "HealthCheck": (HealthCheckConfig, False),
        "IpAddressType": (str, False),
        "LambdaEventStructureVersion": (str, False),
        "Port": (integer, False),
        "Protocol": (str, False),
        "ProtocolVersion": (str, False),
        "VpcIdentifier": (str, False),
    }


class TargetGroup(AWSObject):
    """
    `TargetGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-vpclattice-targetgroup.html>`__
    """

    resource_type = "AWS::VpcLattice::TargetGroup"

    props: PropsDictType = {
        "Config": (TargetGroupConfig, False),
        "Name": (str, False),
        "Tags": (Tags, False),
        "Targets": ([Target], False),
        "Type": (str, True),
    }



import unittest

from troposphere import AWSHelperFn, Parameter, Template
from troposphere.sqs import Queue


class TestServerless(unittest.TestCase):
    def test_transform(self):
        t = Template()
        t.set_version("2010-09-09")
        t.set_transform("AWS::LanguageExtensions")

        self.assertEqual(
            t.to_dict(),
            {
                "AWSTemplateFormatVersion": "2010-09-09",
                "Transform": "AWS::LanguageExtensions",
                "Resources": {},
            },
        )

    def test_length_function(self):
        class Length(AWSHelperFn):
            def __init__(self, data: object) -> None:
                self.data = {"Fn::Length": data}

        t = Template()
        t.set_version("2010-09-09")
        t.set_transform("AWS::LanguageExtensions")

        queue_list = t.add_parameter(Parameter("QueueList", Type="CommaDelimitedList"))
        queue_name = t.add_parameter(
            Parameter(
                "QueueNameParam", Description="Name for your SQS queue", Type="String"
            )
        )

        t.add_resource(
            Queue(
                "Queue",
                QueueName=queue_name.ref(),
                DelaySeconds=Length(queue_list.ref()),
            )
        )

        self.assertEqual(
            t.to_dict(),
            {
                "AWSTemplateFormatVersion": "2010-09-09",
                "Transform": "AWS::LanguageExtensions",
                "Parameters": {
                    "QueueList": {"Type": "CommaDelimitedList"},
                    "QueueNameParam": {
                        "Description": "Name for your SQS queue",
                        "Type": "String",
                    },
                },
                "Resources": {
                    "Queue": {
                        "Type": "AWS::SQS::Queue",
                        "Properties": {
                            "QueueName": {"Ref": "QueueNameParam"},
                            "DelaySeconds": {"Fn::Length": {"Ref": "QueueList"}},
                        },
                    }
                },
            },
        )



from troposphere import (
    And,
    Condition,
    Equals,
    If,
    Not,
    Or,
    Parameter,
    Ref,
    Template,
    ec2,
)

parameters = {
    "One": Parameter(
        "One",
        Type="String",
    ),
    "Two": Parameter(
        "Two",
        Type="String",
    ),
    "Three": Parameter(
        "Three",
        Type="String",
    ),
    "Four": Parameter(
        "Four",
        Type="String",
    ),
    "SshKeyName": Parameter(
        "SshKeyName",
        Type="String",
    ),
}

conditions = {
    "OneEqualsFoo": Equals(Ref("One"), "Foo"),
    "NotOneEqualsFoo": Not(Condition("OneEqualsFoo")),
    "BarEqualsTwo": Equals("Bar", Ref("Two")),
    "ThreeEqualsFour": Equals(Ref("Three"), Ref("Four")),
    "OneEqualsFooOrBarEqualsTwo": Or(
        Condition("OneEqualsFoo"), Condition("BarEqualsTwo")
    ),
    "OneEqualsFooAndNotBarEqualsTwo": And(
        Condition("OneEqualsFoo"), Not(Condition("BarEqualsTwo"))
    ),
    "OneEqualsFooAndBarEqualsTwoAndThreeEqualsPft": And(
        Condition("OneEqualsFoo"),
        Condition("BarEqualsTwo"),
        Equals(Ref("Three"), "Pft"),
    ),
    "OneIsQuzAndThreeEqualsFour": And(
        Equals(Ref("One"), "Quz"), Condition("ThreeEqualsFour")
    ),
    "LaunchInstance": And(
        Condition("OneEqualsFoo"),
        Condition("NotOneEqualsFoo"),
        Condition("BarEqualsTwo"),
        Condition("OneEqualsFooAndNotBarEqualsTwo"),
        Condition("OneIsQuzAndThreeEqualsFour"),
    ),
    "LaunchWithGusto": And(Condition("LaunchInstance"), Equals(Ref("One"), "Gusto")),
}

resources = {
    "Ec2Instance": ec2.Instance(
        "Ec2Instance",
        Condition="LaunchInstance",
        ImageId=If("ConditionNameEqualsFoo", "ami-12345678", "ami-87654321"),
        InstanceType="t1.micro",
        KeyName=Ref("SshKeyName"),
        SecurityGroups=["default"],
    )
}

t = Template()

for p in parameters.values():
    t.add_parameter(p)
for k in conditions:
    t.add_condition(k, conditions[k])
for r in resources.values():
    t.add_resource(r)

print(t.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_logginglevel(slackchannelconfiguration_logginglevel):
    """
    Validate LoggingLevel for SlackChannelConfiguration
    Property: SlackChannelConfiguration.LoggingLevel
    """

    VALID_SLACKCHANNELCONFIGURATION_LOGGINGLEVEL = ("ERROR", "INFO", "NODE")

    if (
        slackchannelconfiguration_logginglevel
        not in VALID_SLACKCHANNELCONFIGURATION_LOGGINGLEVEL
    ):
        raise ValueError(
            "SlackChannelConfiguration LoggingLevel must be one of: %s"
            % ", ".join(VALID_SLACKCHANNELCONFIGURATION_LOGGINGLEVEL)
        )
    return slackchannelconfiguration_logginglevel



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer
from .validators.opsworks import (
    validate_block_device_mapping,
    validate_data_source_type,
    validate_json_checker,
    validate_stack,
    validate_tags_or_list,
    validate_volume_configuration,
    validate_volume_type,
)


class DataSource(AWSProperty):
    """
    `DataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-datasource.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "DatabaseName": (str, False),
        "Type": (validate_data_source_type, False),
    }


class Environment(AWSProperty):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-environment.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Secure": (boolean, False),
        "Value": (str, True),
    }


class Source(AWSProperty):
    """
    `Source <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html>`__
    """

    props: PropsDictType = {
        "Password": (str, False),
        "Revision": (str, False),
        "SshKey": (str, False),
        "Type": (str, False),
        "Url": (str, False),
        "Username": (str, False),
    }


class SslConfiguration(AWSProperty):
    """
    `SslConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-sslconfiguration.html>`__
    """

    props: PropsDictType = {
        "Certificate": (str, False),
        "Chain": (str, False),
        "PrivateKey": (str, False),
    }


class App(AWSObject):
    """
    `App <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html>`__
    """

    resource_type = "AWS::OpsWorks::App"

    props: PropsDictType = {
        "AppSource": (Source, False),
        "Attributes": (dict, False),
        "DataSources": ([DataSource], False),
        "Description": (str, False),
        "Domains": ([str], False),
        "EnableSsl": (boolean, False),
        "Environment": ([Environment], False),
        "Name": (str, True),
        "Shortname": (str, False),
        "SslConfiguration": (SslConfiguration, False),
        "StackId": (str, True),
        "Type": (str, True),
    }


class ElasticLoadBalancerAttachment(AWSObject):
    """
    `ElasticLoadBalancerAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-elbattachment.html>`__
    """

    resource_type = "AWS::OpsWorks::ElasticLoadBalancerAttachment"

    props: PropsDictType = {
        "ElasticLoadBalancerName": (str, True),
        "LayerId": (str, True),
    }


class EbsBlockDevice(AWSProperty):
    """
    `EbsBlockDevice <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-ebsblockdevice.html>`__
    """

    props: PropsDictType = {
        "DeleteOnTermination": (boolean, False),
        "Iops": (integer, False),
        "SnapshotId": (str, False),
        "VolumeSize": (integer, False),
        "VolumeType": (str, False),
    }


class BlockDeviceMapping(AWSProperty):
    """
    `BlockDeviceMapping <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-blockdevicemapping.html>`__
    """

    props: PropsDictType = {
        "DeviceName": (str, False),
        "Ebs": (EbsBlockDevice, False),
        "NoDevice": (str, False),
        "VirtualName": (str, False),
    }

    def validate(self):
        validate_block_device_mapping(self)


class TimeBasedAutoScaling(AWSProperty):
    """
    `TimeBasedAutoScaling <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html>`__
    """

    props: PropsDictType = {
        "Friday": (dict, False),
        "Monday": (dict, False),
        "Saturday": (dict, False),
        "Sunday": (dict, False),
        "Thursday": (dict, False),
        "Tuesday": (dict, False),
        "Wednesday": (dict, False),
    }


class Instance(AWSObject):
    """
    `Instance <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html>`__
    """

    resource_type = "AWS::OpsWorks::Instance"

    props: PropsDictType = {
        "AgentVersion": (str, False),
        "AmiId": (str, False),
        "Architecture": (str, False),
        "AutoScalingType": (str, False),
        "AvailabilityZone": (str, False),
        "BlockDeviceMappings": ([BlockDeviceMapping], False),
        "EbsOptimized": (boolean, False),
        "ElasticIps": ([str], False),
        "Hostname": (str, False),
        "InstallUpdatesOnBoot": (boolean, False),
        "InstanceType": (str, True),
        "LayerIds": ([str], True),
        "Os": (str, False),
        "RootDeviceType": (str, False),
        "SshKeyName": (str, False),
        "StackId": (str, True),
        "SubnetId": (str, False),
        "Tenancy": (str, False),
        "TimeBasedAutoScaling": (TimeBasedAutoScaling, False),
        "VirtualizationType": (str, False),
        "Volumes": ([str], False),
    }


class ShutdownEventConfiguration(AWSProperty):
    """
    `ShutdownEventConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-lifecycleeventconfiguration-shutdowneventconfiguration.html>`__
    """

    props: PropsDictType = {
        "DelayUntilElbConnectionsDrained": (boolean, False),
        "ExecutionTimeout": (integer, False),
    }


class LifeCycleConfiguration(AWSProperty):
    """
    `LifeCycleConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-lifecycleeventconfiguration.html>`__
    """

    props: PropsDictType = {
        "ShutdownEventConfiguration": (ShutdownEventConfiguration, False),
    }


class AutoScalingThresholds(AWSProperty):
    """
    `AutoScalingThresholds <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html>`__
    """

    props: PropsDictType = {
        "CpuThreshold": (double, False),
        "IgnoreMetricsTime": (integer, False),
        "InstanceCount": (integer, False),
        "LoadThreshold": (double, False),
        "MemoryThreshold": (double, False),
        "ThresholdsWaitTime": (integer, False),
    }


class LoadBasedAutoScaling(AWSProperty):
    """
    `LoadBasedAutoScaling <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling.html>`__
    """

    props: PropsDictType = {
        "DownScaling": (AutoScalingThresholds, False),
        "Enable": (boolean, False),
        "UpScaling": (AutoScalingThresholds, False),
    }


class Recipes(AWSProperty):
    """
    `Recipes <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-recipes.html>`__
    """

    props: PropsDictType = {
        "Configure": ([str], False),
        "Deploy": ([str], False),
        "Setup": ([str], False),
        "Shutdown": ([str], False),
        "Undeploy": ([str], False),
    }


class VolumeConfiguration(AWSProperty):
    """
    `VolumeConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html>`__
    """

    props: PropsDictType = {
        "Encrypted": (boolean, False),
        "Iops": (integer, False),
        "MountPoint": (str, False),
        "NumberOfDisks": (integer, False),
        "RaidLevel": (integer, False),
        "Size": (integer, False),
        "VolumeType": (validate_volume_type, False),
    }

    def validate(self):
        validate_volume_configuration(self)


class Layer(AWSObject):
    """
    `Layer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html>`__
    """

    resource_type = "AWS::OpsWorks::Layer"

    props: PropsDictType = {
        "Attributes": (dict, False),
        "AutoAssignElasticIps": (boolean, True),
        "AutoAssignPublicIps": (boolean, True),
        "CustomInstanceProfileArn": (str, False),
        "CustomJson": (validate_json_checker, False),
        "CustomRecipes": (Recipes, False),
        "CustomSecurityGroupIds": ([str], False),
        "EnableAutoHealing": (boolean, True),
        "InstallUpdatesOnBoot": (boolean, False),
        "LifecycleEventConfiguration": (LifeCycleConfiguration, False),
        "LoadBasedAutoScaling": (LoadBasedAutoScaling, False),
        "Name": (str, True),
        "Packages": ([str], False),
        "Shortname": (str, True),
        "StackId": (str, True),
        "Tags": (Tags, False),
        "Type": (str, True),
        "UseEbsOptimizedInstances": (boolean, False),
        "VolumeConfigurations": ([VolumeConfiguration], False),
    }


class ChefConfiguration(AWSProperty):
    """
    `ChefConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-chefconfiguration.html>`__
    """

    props: PropsDictType = {
        "BerkshelfVersion": (str, False),
        "ManageBerkshelf": (boolean, False),
    }


class ElasticIp(AWSProperty):
    """
    `ElasticIp <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-elasticip.html>`__
    """

    props: PropsDictType = {
        "Ip": (str, True),
        "Name": (str, False),
    }


class RdsDbInstance(AWSProperty):
    """
    `RdsDbInstance <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-rdsdbinstance.html>`__
    """

    props: PropsDictType = {
        "DbPassword": (str, True),
        "DbUser": (str, True),
        "RdsDbInstanceArn": (str, True),
    }


class StackConfigurationManager(AWSProperty):
    """
    `StackConfigurationManager <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-stackconfigmanager.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Version": (str, False),
    }


class Stack(AWSObject):
    """
    `Stack <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html>`__
    """

    resource_type = "AWS::OpsWorks::Stack"

    props: PropsDictType = {
        "AgentVersion": (str, False),
        "Attributes": (dict, False),
        "ChefConfiguration": (ChefConfiguration, False),
        "CloneAppIds": ([str], False),
        "ClonePermissions": (boolean, False),
        "ConfigurationManager": (StackConfigurationManager, False),
        "CustomCookbooksSource": (Source, False),
        "CustomJson": (validate_json_checker, False),
        "DefaultAvailabilityZone": (str, False),
        "DefaultInstanceProfileArn": (str, True),
        "DefaultOs": (str, False),
        "DefaultRootDeviceType": (str, False),
        "DefaultSshKeyName": (str, False),
        "DefaultSubnetId": (str, False),
        "EcsClusterArn": (str, False),
        "ElasticIps": ([ElasticIp], False),
        "HostnameTheme": (str, False),
        "Name": (str, True),
        "RdsDbInstances": ([RdsDbInstance], False),
        "ServiceRoleArn": (str, True),
        "SourceStackId": (str, False),
        "Tags": (validate_tags_or_list, False),
        "UseCustomCookbooks": (boolean, False),
        "UseOpsworksSecurityGroups": (boolean, False),
        "VpcId": (str, False),
    }

    def validate(self):
        validate_stack(self)


class UserProfile(AWSObject):
    """
    `UserProfile <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html>`__
    """

    resource_type = "AWS::OpsWorks::UserProfile"

    props: PropsDictType = {
        "AllowSelfManagement": (boolean, False),
        "IamUserArn": (str, True),
        "SshPublicKey": (str, False),
        "SshUsername": (str, False),
    }


class Volume(AWSObject):
    """
    `Volume <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html>`__
    """

    resource_type = "AWS::OpsWorks::Volume"

    props: PropsDictType = {
        "Ec2VolumeId": (str, True),
        "MountPoint": (str, False),
        "Name": (str, False),
        "StackId": (str, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags


class AttributeMapItems(AWSProperty):
    """
    `AttributeMapItems <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-finspace-environment-attributemapitems.html>`__
    """

    props: PropsDictType = {
        "Key": (str, False),
        "Value": (str, False),
    }


class FederationParameters(AWSProperty):
    """
    `FederationParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-finspace-environment-federationparameters.html>`__
    """

    props: PropsDictType = {
        "ApplicationCallBackURL": (str, False),
        "AttributeMap": ([AttributeMapItems], False),
        "FederationProviderName": (str, False),
        "FederationURN": (str, False),
        "SamlMetadataDocument": (str, False),
        "SamlMetadataURL": (str, False),
    }


class SuperuserParameters(AWSProperty):
    """
    `SuperuserParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-finspace-environment-superuserparameters.html>`__
    """

    props: PropsDictType = {
        "EmailAddress": (str, False),
        "FirstName": (str, False),
        "LastName": (str, False),
    }


class Environment(AWSObject):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-finspace-environment.html>`__
    """

    resource_type = "AWS::FinSpace::Environment"

    props: PropsDictType = {
        "Description": (str, False),
        "FederationMode": (str, False),
        "FederationParameters": (FederationParameters, False),
        "KmsKeyId": (str, False),
        "Name": (str, True),
        "SuperuserParameters": (SuperuserParameters, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, PropsDictType, Tags
from .validators import boolean, integer
from .validators.kms import (
    key_usage_type,
    policytypes,
    validate_pending_window_in_days,
    validate_tags_or_list,
)


class Alias(AWSObject):
    """
    `Alias <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-alias.html>`__
    """

    resource_type = "AWS::KMS::Alias"

    props: PropsDictType = {
        "AliasName": (str, True),
        "TargetKeyId": (str, True),
    }


class Key(AWSObject):
    """
    `Key <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html>`__
    """

    resource_type = "AWS::KMS::Key"

    props: PropsDictType = {
        "BypassPolicyLockoutSafetyCheck": (boolean, False),
        "Description": (str, False),
        "EnableKeyRotation": (boolean, False),
        "Enabled": (boolean, False),
        "KeyPolicy": (policytypes, False),
        "KeySpec": (str, False),
        "KeyUsage": (key_usage_type, False),
        "MultiRegion": (boolean, False),
        "Origin": (str, False),
        "PendingWindowInDays": (validate_pending_window_in_days, False),
        "Tags": (validate_tags_or_list, False),
    }


class ReplicaKey(AWSObject):
    """
    `ReplicaKey <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-replicakey.html>`__
    """

    resource_type = "AWS::KMS::ReplicaKey"

    props: PropsDictType = {
        "Description": (str, False),
        "Enabled": (boolean, False),
        "KeyPolicy": (dict, True),
        "PendingWindowInDays": (integer, False),
        "PrimaryKeyArn": (str, True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from ..type_defs.compat import Final
from . import exactly_one, mutually_exclusive

KEY_ONLY: Final = "KEY_ONLY"
VALUE_ONLY: Final = "VALUE_ONLY"
KEY_AND_VALUE: Final = "KEY_AND_VALUE"


def deployment_option_validator(x):
    """
    Property: DeploymentStyle.DeploymentOption
    """
    valid_values = ["WITH_TRAFFIC_CONTROL", "WITHOUT_TRAFFIC_CONTROL"]
    if x not in valid_values:
        raise ValueError(
            "Deployment Option value must be one of: %s" % ", ".join(valid_values)
        )
    return x


def deployment_type_validator(x):
    """
    Property: DeploymentStyle.DeploymentType
    """
    valid_values = ["IN_PLACE", "BLUE_GREEN"]
    if x not in valid_values:
        raise ValueError(
            "Deployment Type value must be one of: %s" % ", ".join(valid_values)
        )
    return x


def validate_load_balancer_info(self):
    """
    Class: LoadBalancerInfo
    """
    conds = ["ElbInfoList", "TargetGroupInfoList", "TargetGroupPairInfoList"]
    exactly_one(self.__class__.__name__, self.properties, conds)


def validate_deployment_group(self):
    """
    Class: DeploymentGroup
    """
    ec2_conds = ["EC2TagFilters", "Ec2TagSet"]
    onPremises_conds = ["OnPremisesInstanceTagFilters", "OnPremisesTagSet"]
    mutually_exclusive(self.__class__.__name__, self.properties, ec2_conds)
    mutually_exclusive(self.__class__.__name__, self.properties, onPremises_conds)



import unittest

import troposphere.rds as rds
import troposphere.validators.rds as vrds
from troposphere import If, Parameter, Ref

AWS_NO_VALUE = "AWS::NoValue"


class TestRDS(unittest.TestCase):
    def test_it_allows_an_rds_instance_created_from_a_snapshot(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=100,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
            DBSnapshotIdentifier="SomeSnapshotIdentifier",
        )

        rds_instance.to_dict()

    def test_it_allows_an_rds_instance_with_master_username_and_password(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=1,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
            MasterUsername="SomeUsername",
            MasterUserPassword="SomePassword",
        )

        rds_instance.to_dict()

    def test_it_rds_instances_require_either_a_snapshot_or_credentials(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=1,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
        )

        with self.assertRaisesRegex(
            ValueError,
            r"Either \(MasterUsername and either "
            r"MasterUserPassword or ManageMasterUserPassword\) or"
            r" DBSnapshotIdentifier are required",
        ):
            rds_instance.to_dict()

    def test_it_rds_credentials_using_masteruserpassword(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            Engine="MySQL",
            MasterUsername="user",
            MasterUserPassword="password",
        )
        rds_instance.to_dict()

    def test_it_rds_credentials_using_managemasteruserpassword(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            Engine="MySQL",
            MasterUsername="user",
            ManageMasterUserPassword=True,
        )
        rds_instance.to_dict()

    def test_it_rds_masteruserpassword_and_managemasteruserpassword_mutually_exclusive(
        self,
    ):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            Engine="MySQL",
            MasterUsername="user",
            MasterUserPassword="password",
            ManageMasterUserPassword=True,
        )

        with self.assertRaisesRegex(
            ValueError,
            r"Both MasterUserPassword and ManageMasterUserPassword cannot be set simultaneously.",
        ):
            rds_instance.to_dict()

    def test_it_allows_an_rds_replica(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=1,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
            SourceDBInstanceIdentifier="SomeSourceDBInstanceIdentifier",
        )

        rds_instance.to_dict()

    def test_replica_settings_are_inherited(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=1,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
            SourceDBInstanceIdentifier="SomeSourceDBInstanceIdentifier",
            BackupRetentionPeriod="1",
            DBName="SomeName",
            MasterUsername="SomeUsername",
            MasterUserPassword="SomePassword",
            PreferredBackupWindow="10:00-11:00",
            MultiAZ=True,
            DBSnapshotIdentifier="SomeDBSnapshotIdentifier",
        )

        with self.assertRaisesRegex(
            ValueError,
            "DBName, DBSnapshotIdentifier, "
            "MasterUserPassword, MasterUsername, "
            "MultiAZ, PreferredBackupWindow "
            "properties can't be provided when "
            "SourceDBInstanceIdentifier is present "
            "AWS::RDS::DBInstance.",
        ):
            rds_instance.to_dict()

    def test_it_rds_instances_require_encryption_if_kms_key_provided(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=1,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
            MasterUsername="SomeUsername",
            MasterUserPassword="SomePassword",
            KmsKeyId="arn:aws:kms:us-east-1:123456789012:key/"
            "12345678-1234-1234-1234-123456789012",
        )

        with self.assertRaisesRegex(
            ValueError, "If KmsKeyId is provided, StorageEncrypted is required"
        ):
            rds_instance.to_dict()

    def test_it_allows_an_rds_instance_with_iops(self):
        rds_instance = rds.DBInstance(
            "SomeTitle",
            AllocatedStorage=200,
            DBInstanceClass="db.m1.small",
            Engine="MySQL",
            MasterUsername="SomeUsername",
            MasterUserPassword="SomePassword",
            StorageType="io1",
            Iops=2000,
        )

        rds_instance.to_dict()

    def test_optiongroup(self):
        rds_optiongroup = rds.OptionGroup(
            "OracleOptionGroup",
            EngineName="oracle-ee",
            MajorEngineVersion="12.1",
            OptionGroupDescription="A test option group",
            OptionConfigurations=[
                rds.OptionConfiguration(
                    DBSecurityGroupMemberships=["default"],
                    OptionName="OEM",
                    Port="5500",
                ),
                rds.OptionConfiguration(
                    OptionName="APEX",
                ),
            ],
        )
        rds_optiongroup.to_dict()

    def test_fail_az_and_multiaz(self):
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            AllocatedStorage=10,
            DBInstanceClass="db.m1.small",
            Engine="postgres",
            AvailabilityZone="us-east-1",
            MultiAZ=True,
        )
        with self.assertRaisesRegex(ValueError, "if MultiAZ is set to "):
            i.to_dict()
        i.MultiAZ = "false"
        i.to_dict()
        i.MultiAZ = "true"
        with self.assertRaisesRegex(ValueError, "if MultiAZ is set to "):
            i.to_dict()

        i.MultiAZ = Ref(AWS_NO_VALUE)
        i.to_dict()

    def test_az_and_multiaz_funcs(self):
        db_az = "us-east-1"
        db_multi_az = Parameter("dbmultiaz", Type="String")
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            AllocatedStorage=10,
            DBInstanceClass="db.m1.small",
            Engine="postgres",
            AvailabilityZone=If("db_az", Ref(db_az), Ref(AWS_NO_VALUE)),
            MultiAZ=Ref(db_multi_az),
        )
        i.validate()

    def test_allocated_storage_parameter_oracle(self):
        db_az = "us-east-1"
        allocated_storage = Parameter("allocatedstorage", Type="int")
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            AllocatedStorage=Ref(allocated_storage),
            DBInstanceClass="db.m1.small",
            Engine="oracle-se1",
            StorageType="gp3",
            AvailabilityZone=If("db_az", Ref(db_az), Ref(AWS_NO_VALUE)),
        )
        i.validate()

    def test_allocated_storage_parameter_postgres(self):
        db_az = "us-east-1"
        allocated_storage = Parameter("allocatedstorage", Type="int")
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            AllocatedStorage=Ref(allocated_storage),
            DBInstanceClass="db.m1.small",
            Engine="postgres",
            StorageType="gp3",
            AvailabilityZone=If("db_az", Ref(db_az), Ref(AWS_NO_VALUE)),
        )
        i.validate()

    def test_io1_storage_type_and_iops(self):
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            AllocatedStorage=10,
            DBInstanceClass="db.m1.small",
            Engine="postgres",
            StorageType="io1",
        )
        with self.assertRaisesRegex(ValueError, "Must specify Iops if "):
            i.to_dict()

    def test_storage_to_iops_ratio(self):
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            DBInstanceClass="db.m1.small",
            Engine="postgres",
            StorageType="io1",
            Iops=6000,
            AllocatedStorage=10,
        )
        with self.assertRaisesRegex(ValueError, " must be at least 100 "):
            i.to_dict()

        i.AllocatedStorage = 100
        with self.assertRaisesRegex(ValueError, " must be no less than 1/50th "):
            i.to_dict()

        i.Iops = 5000
        i.to_dict()

    def test_storage_sqlserver(self):
        i = rds.DBInstance(
            "NoAZAndMultiAZ",
            MasterUsername="myuser",
            MasterUserPassword="mypassword",
            DBInstanceClass="db.m1.small",
            Engine="sqlserver-ee",
            StorageType="io1",
            Iops=1000,
            AllocatedStorage=10,
        )
        with self.assertRaisesRegex(ValueError, " must be at least 20 "):
            i.to_dict()

        i.AllocatedStorage = 20
        i.to_dict()

    def test_snapshot(self):
        i = rds.DBInstance(
            "MyDB",
            DBName="test",
            AllocatedStorage=25,
            DBInstanceClass="db.m4.large",
            DBSubnetGroupName="default",
            DBSnapshotIdentifier="id",
        )
        i.to_dict()

    def test_snapshot_and_engine(self):
        i = rds.DBInstance(
            "MyDB",
            DBName="test",
            AllocatedStorage=25,
            DBInstanceClass="db.m4.large",
            DBSubnetGroupName="default",
            DBSnapshotIdentifier="id",
            Engine="postgres",
        )
        i.to_dict()

    def test_no_snapshot_or_engine(self):
        i = rds.DBInstance(
            "MyDB",
            DBName="test",
            AllocatedStorage=25,
            DBInstanceClass="db.m4.large",
            DBSubnetGroupName="default",
        )
        with self.assertRaisesRegex(ValueError, "Resource Engine is required"):
            i.to_dict()


class TestRDSValidators(unittest.TestCase):
    def test_validate_iops(self):
        with self.assertRaises(ValueError):
            vrds.validate_iops(500)
        vrds.validate_iops(2000)
        vrds.validate_iops(0)

    def test_validate_storage_type(self):
        vrds.validate_storage_type("standard")

        with self.assertRaises(ValueError):
            vrds.validate_storage_type("bad_storage_type")

    def test_validate_engine(self):
        vrds.validate_engine("postgres")

        with self.assertRaises(ValueError):
            vrds.validate_engine("bad_engine")

    def test_validate_engine_mode(self):
        vrds.validate_engine_mode("provisioned")

        with self.assertRaises(ValueError):
            vrds.validate_engine_mode("bad_engine")

    def test_validate_license_model(self):
        vrds.validate_license_model("postgresql-license")

        with self.assertRaises(ValueError):
            vrds.validate_license_model("bad_license_model")

    def test_validate_backup_window(self):
        good_windows = ("10:00-11:00", "22:00-06:00")
        for w in good_windows:
            vrds.validate_backup_window(w)

        bad_format = ("bad_backup_window", "28:11-10:00", "10:00-28:11")
        for w in bad_format:
            with self.assertRaisesRegex(ValueError, "must be in the format"):
                vrds.validate_backup_window(w)

        with self.assertRaisesRegex(ValueError, "must be at least 30 "):
            vrds.validate_backup_window("10:00-10:10")

    def test_validate_maintenance_window(self):
        good_windows = (
            "Mon:10:00-Mon:16:30",
            "Mon:10:00-Wed:10:00",
            "Sun:16:00-Mon:11:00",
        )

        for w in good_windows:
            vrds.validate_maintenance_window(w)

        bad_format = ("bad_mainteance", "Mon:10:00-Tue:28:00", "10:00-22:00")
        for w in bad_format:
            with self.assertRaisesRegex(ValueError, "must be in the format"):
                vrds.validate_maintenance_window(w)

        bad_days = ("Boo:10:00-Woo:10:30", "Boo:10:00-Tue:10:30", "Mon:10:00-Boo:10:30")
        for w in bad_days:
            with self.assertRaisesRegex(ValueError, " day part of ranges "):
                vrds.validate_maintenance_window(w)

        with self.assertRaisesRegex(ValueError, "must be at least 30 "):
            vrds.validate_maintenance_window("Mon:10:00-Mon:10:10")

    def test_validate_backup_retention_period(self):
        for d in (1, 10, 15, 35):
            vrds.validate_backup_retention_period(d)

        with self.assertRaisesRegex(ValueError, " cannot be larger than 35 "):
            vrds.validate_backup_retention_period(40)

        vrds.validate_backup_retention_period(10)

    def test_validate_capacity(self):
        vrds.validate_capacity(64)

        with self.assertRaises(ValueError):
            vrds.validate_capacity(3)

        with self.assertRaises(ValueError):
            vrds.validate_capacity(100001)

    def test_v2_validate_capacity(self):
        vrds.validate_v2_capacity(64)
        vrds.validate_v2_capacity(0.5)

        with self.assertRaises(ValueError):
            vrds.validate_v2_capacity(129)

        with self.assertRaises(ValueError):
            vrds.validate_v2_capacity(1.1)

        with self.assertRaises(ValueError):
            vrds.validate_v2_max_capacity(0.5)



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_clusteringkeycolumn_orderby(clusteringkeycolumn_orderby):
    """
    Property: ClusteringKeyColumn.OrderBy
    """

    VALID_CLUSTERINGKEYCOLUMN_ORDERBY = ("ASC", "DESC")

    if clusteringkeycolumn_orderby not in VALID_CLUSTERINGKEYCOLUMN_ORDERBY:
        raise ValueError(
            "ClusteringKeyColumn OrderBy must be one of: %s"
            % ", ".join(VALID_CLUSTERINGKEYCOLUMN_ORDERBY)
        )
    return clusteringkeycolumn_orderby


def validate_billingmode_mode(billingmode_mode):
    """
    Property: BillingMode.Mode
    """

    VALID_BILLINGMODE_MODE = ("ON_DEMAND", "PROVISIONED")

    if billingmode_mode not in VALID_BILLINGMODE_MODE:
        raise ValueError(
            "BillingMode Mode must be one of: %s" % ", ".join(VALID_BILLINGMODE_MODE)
        )
    return billingmode_mode



import unittest

import troposphere.cloudwatch as cloudwatch
from troposphere.cloudwatch import Dashboard


class TestModel(unittest.TestCase):
    def test_dashboard(self):
        # Check valid json dashboard string
        dashboard = Dashboard(
            "dashboard",
            DashboardBody='{"a": "b"}',
        )
        dashboard.validate()

        # Check invalid json dashboard string
        dashboard = Dashboard(
            "dashboard",
            DashboardBody='{"a: "b"}',
        )
        with self.assertRaises(ValueError):
            dashboard.validate()

        # Check accepting dict and converting to string in validate
        d = {"c": "d"}
        dashboard = Dashboard("dashboard", DashboardBody=d)
        dashboard.validate()
        self.assertEqual(dashboard.properties["DashboardBody"], '{"c": "d"}')

        # Check invalid Dashboard type
        with self.assertRaises(TypeError):
            dashboard = Dashboard("dashboard", DashboardBody=1)


class TestCloudWatchValidators(unittest.TestCase):
    def test_validate_units(self):
        cloudwatch.validate_unit("Bytes/Second")
        for bad_unit in ["Minutes", "Bytes/Minute", "Bits/Hour", ""]:
            with self.assertRaisesRegex(ValueError, "must be one of"):
                cloudwatch.validate_unit(bad_unit)

    def test_validate_treat_missing_data(self):
        cloudwatch.validate_treat_missing_data("missing")
        for bad_value in ["exists", "notMissing", ""]:
            with self.assertRaisesRegex(ValueError, "must be one of"):
                cloudwatch.validate_treat_missing_data(bad_value)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from .. import AWSHelperFn


def validate_trigger(self):
    """
    Class: Trigger
    """
    valid = [
        "all",
        "createReference",
        "deleteReference",
        "updateReference",
    ]
    events = self.properties.get("Events")
    if events and not isinstance(events, AWSHelperFn):
        if "all" in events and len(events) != 1:
            raise ValueError("Trigger events: all must be used alone")
        else:
            for e in events:
                if e not in valid and not isinstance(e, AWSHelperFn):
                    raise ValueError("Trigger: invalid event %s" % e)



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, double


class NetworkAccessControl(AWSProperty):
    """
    `NetworkAccessControl <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-grafana-workspace-networkaccesscontrol.html>`__
    """

    props: PropsDictType = {
        "PrefixListIds": ([str], False),
        "VpceIds": ([str], False),
    }


class AssertionAttributes(AWSProperty):
    """
    `AssertionAttributes <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-grafana-workspace-assertionattributes.html>`__
    """

    props: PropsDictType = {
        "Email": (str, False),
        "Groups": (str, False),
        "Login": (str, False),
        "Name": (str, False),
        "Org": (str, False),
        "Role": (str, False),
    }


class IdpMetadata(AWSProperty):
    """
    `IdpMetadata <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-grafana-workspace-idpmetadata.html>`__
    """

    props: PropsDictType = {
        "Url": (str, False),
        "Xml": (str, False),
    }


class RoleValues(AWSProperty):
    """
    `RoleValues <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-grafana-workspace-rolevalues.html>`__
    """

    props: PropsDictType = {
        "Admin": ([str], False),
        "Editor": ([str], False),
    }


class SamlConfiguration(AWSProperty):
    """
    `SamlConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-grafana-workspace-samlconfiguration.html>`__
    """

    props: PropsDictType = {
        "AllowedOrganizations": ([str], False),
        "AssertionAttributes": (AssertionAttributes, False),
        "IdpMetadata": (IdpMetadata, True),
        "LoginValidityDuration": (double, False),
        "RoleValues": (RoleValues, False),
    }


class VpcConfiguration(AWSProperty):
    """
    `VpcConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-grafana-workspace-vpcconfiguration.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupIds": ([str], True),
        "SubnetIds": ([str], True),
    }


class Workspace(AWSObject):
    """
    `Workspace <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-grafana-workspace.html>`__
    """

    resource_type = "AWS::Grafana::Workspace"

    props: PropsDictType = {
        "AccountAccessType": (str, True),
        "AuthenticationProviders": ([str], True),
        "ClientToken": (str, False),
        "DataSources": ([str], False),
        "Description": (str, False),
        "GrafanaVersion": (str, False),
        "Name": (str, False),
        "NetworkAccessControl": (NetworkAccessControl, False),
        "NotificationDestinations": ([str], False),
        "OrganizationRoleName": (str, False),
        "OrganizationalUnits": ([str], False),
        "PermissionType": (str, True),
        "PluginAdminEnabled": (boolean, False),
        "RoleArn": (str, False),
        "SamlConfiguration": (SamlConfiguration, False),
        "StackSetName": (str, False),
        "VpcConfiguration": (VpcConfiguration, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def processor_type_validator(x):
    """
    Property: Processor.Type
    """
    valid_types = [
        "Lambda",
        "MetadataExtraction",
        "RecordDeAggregation",
        "AppendDelimiterToRecord",
    ]
    if x not in valid_types:
        raise ValueError("Type must be one of: %s" % ", ".join(valid_types))
    return x


def delivery_stream_type_validator(x):
    """
    Property: DeliveryStream.DeliveryStreamType
    """
    valid_types = ["DirectPut", "KinesisStreamAsSource"]
    if x not in valid_types:
        raise ValueError(
            "DeliveryStreamType must be one of: %s" % ", ".join(valid_types)
        )
    return x


def index_rotation_period_validator(x):
    """
    Property: ElasticsearchDestinationConfiguration.IndexRotationPeriod
    """
    valid_types = ["NoRotation", "OneHour", "OneDay", "OneWeek", "OneMonth"]
    if x not in valid_types:
        raise ValueError(
            "IndexRotationPeriod must be one of: %s" % ", ".join(valid_types)
        )
    return x


def s3_backup_mode_elastic_search_validator(x):
    """
    Property: ElasticsearchDestinationConfiguration.S3BackupMode
    """
    valid_types = ["FailedDocumentsOnly", "AllDocuments"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" % ", ".join(valid_types))
    return x


def s3_backup_mode_extended_s3_validator(x):
    """
    Property: ExtendedS3DestinationConfiguration.S3BackupMode
    """
    valid_types = ["Disabled", "Enabled"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" % ", ".join(valid_types))
    return x



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class ArtifactDetails(AWSProperty):
    """
    `ArtifactDetails <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-artifactdetails.html>`__
    """

    props: PropsDictType = {
        "MaximumCount": (integer, True),
        "MinimumCount": (integer, True),
    }


class ConfigurationProperties(AWSProperty):
    """
    `ConfigurationProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html>`__
    """

    props: PropsDictType = {
        "Description": (str, False),
        "Key": (boolean, True),
        "Name": (str, True),
        "Queryable": (boolean, False),
        "Required": (boolean, True),
        "Secret": (boolean, True),
        "Type": (str, False),
    }


class Settings(AWSProperty):
    """
    `Settings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-settings.html>`__
    """

    props: PropsDictType = {
        "EntityUrlTemplate": (str, False),
        "ExecutionUrlTemplate": (str, False),
        "RevisionUrlTemplate": (str, False),
        "ThirdPartyConfigurationUrl": (str, False),
    }


class CustomActionType(AWSObject):
    """
    `CustomActionType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html>`__
    """

    resource_type = "AWS::CodePipeline::CustomActionType"

    props: PropsDictType = {
        "Category": (str, True),
        "ConfigurationProperties": ([ConfigurationProperties], False),
        "InputArtifactDetails": (ArtifactDetails, True),
        "OutputArtifactDetails": (ArtifactDetails, True),
        "Provider": (str, True),
        "Settings": (Settings, False),
        "Tags": (Tags, False),
        "Version": (str, True),
    }


class EncryptionKey(AWSProperty):
    """
    `EncryptionKey <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore-encryptionkey.html>`__
    """

    props: PropsDictType = {
        "Id": (str, True),
        "Type": (str, True),
    }


class ArtifactStore(AWSProperty):
    """
    `ArtifactStore <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore.html>`__
    """

    props: PropsDictType = {
        "EncryptionKey": (EncryptionKey, False),
        "Location": (str, True),
        "Type": (str, True),
    }


class ArtifactStoreMap(AWSProperty):
    """
    `ArtifactStoreMap <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstoremap.html>`__
    """

    props: PropsDictType = {
        "ArtifactStore": (ArtifactStore, True),
        "Region": (str, True),
    }


class DisableInboundStageTransitions(AWSProperty):
    """
    `DisableInboundStageTransitions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-disableinboundstagetransitions.html>`__
    """

    props: PropsDictType = {
        "Reason": (str, True),
        "StageName": (str, True),
    }


class GitTagFilterCriteria(AWSProperty):
    """
    `GitTagFilterCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-triggers-git-tag-filter-criteria.html>`__
    """

    props: PropsDictType = {
        "Excludes": ([str], False),
        "Includes": ([str], False),
    }


class GitPushFilter(AWSProperty):
    """
    `GitPushFilter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-triggers-git-push-filter.html>`__
    """

    props: PropsDictType = {
        "Tags": (GitTagFilterCriteria, False),
    }


class GitConfiguration(AWSProperty):
    """
    `GitConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-triggers-git-configuration.html>`__
    """

    props: PropsDictType = {
        "Push": ([GitPushFilter], False),
        "SourceActionName": (str, True),
    }


class PipelineTriggerDeclaration(AWSProperty):
    """
    `PipelineTriggerDeclaration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-triggers.html>`__
    """

    props: PropsDictType = {
        "GitConfiguration": (GitConfiguration, False),
        "ProviderType": (str, True),
    }


class ActionTypeId(AWSProperty):
    """
    `ActionTypeId <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-actiontypeid.html>`__
    """

    props: PropsDictType = {
        "Category": (str, True),
        "Owner": (str, True),
        "Provider": (str, True),
        "Version": (str, True),
    }


class InputArtifacts(AWSProperty):
    """
    `InputArtifacts <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-inputartifacts.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
    }


class OutputArtifacts(AWSProperty):
    """
    `OutputArtifacts <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-outputartifacts.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
    }


class Actions(AWSProperty):
    """
    `Actions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html>`__
    """

    props: PropsDictType = {
        "ActionTypeId": (ActionTypeId, True),
        "Configuration": (dict, False),
        "InputArtifacts": ([InputArtifacts], False),
        "Name": (str, True),
        "Namespace": (str, False),
        "OutputArtifacts": ([OutputArtifacts], False),
        "Region": (str, False),
        "RoleArn": (str, False),
        "RunOrder": (integer, False),
    }


class Blockers(AWSProperty):
    """
    `Blockers <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-blockers.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "Type": (str, True),
    }


class Stages(AWSProperty):
    """
    `Stages <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages.html>`__
    """

    props: PropsDictType = {
        "Actions": ([Actions], True),
        "Blockers": ([Blockers], False),
        "Name": (str, True),
    }


class VariableDeclaration(AWSProperty):
    """
    `VariableDeclaration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-variables.html>`__
    """

    props: PropsDictType = {
        "DefaultValue": (str, False),
        "Description": (str, False),
        "Name": (str, True),
    }


class Pipeline(AWSObject):
    """
    `Pipeline <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html>`__
    """

    resource_type = "AWS::CodePipeline::Pipeline"

    props: PropsDictType = {
        "ArtifactStore": (ArtifactStore, False),
        "ArtifactStores": ([ArtifactStoreMap], False),
        "DisableInboundStageTransitions": ([DisableInboundStageTransitions], False),
        "Name": (str, False),
        "PipelineType": (str, False),
        "RestartExecutionOnUpdate": (boolean, False),
        "RoleArn": (str, True),
        "Stages": ([Stages], True),
        "Tags": (Tags, False),
        "Triggers": ([PipelineTriggerDeclaration], False),
        "Variables": ([VariableDeclaration], False),
    }


class WebhookAuthConfiguration(AWSProperty):
    """
    `WebhookAuthConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-webhook-webhookauthconfiguration.html>`__
    """

    props: PropsDictType = {
        "AllowedIPRange": (str, False),
        "SecretToken": (str, False),
    }


class WebhookFilterRule(AWSProperty):
    """
    `WebhookFilterRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-webhook-webhookfilterrule.html>`__
    """

    props: PropsDictType = {
        "JsonPath": (str, True),
        "MatchEquals": (str, False),
    }


class Webhook(AWSObject):
    """
    `Webhook <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-webhook.html>`__
    """

    resource_type = "AWS::CodePipeline::Webhook"

    props: PropsDictType = {
        "Authentication": (str, True),
        "AuthenticationConfiguration": (WebhookAuthConfiguration, True),
        "Filters": ([WebhookFilterRule], True),
        "Name": (str, False),
        "RegisterWithThirdParty": (boolean, False),
        "TargetAction": (str, True),
        "TargetPipeline": (str, True),
        "TargetPipelineVersion": (integer, True),
    }



import unittest

from troposphere.route53 import AliasTarget


class TestAliasTarget(unittest.TestCase):
    def test_bucket_template(self):
        AliasTarget("zone", "dnsname", True)
        AliasTarget(hostedzoneid="zone", dnsname="dnsname", evaluatetargethealth=True)
        AliasTarget(HostedZoneId="zone", DNSName="dnsname", EvaluateTargetHealth=True)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer
from .validators.dynamodb import (
    attribute_type_validator,
    billing_mode_validator,
    key_type_validator,
    projection_type_validator,
    table_class_validator,
    validate_table,
)


class AttributeDefinition(AWSProperty):
    """
    `AttributeDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-attributedefinition.html>`__
    """

    props: PropsDictType = {
        "AttributeName": (str, True),
        "AttributeType": (attribute_type_validator, True),
    }


class KeySchema(AWSProperty):
    """
    `KeySchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-keyschema.html>`__
    """

    props: PropsDictType = {
        "AttributeName": (str, True),
        "KeyType": (key_type_validator, True),
    }


class Projection(AWSProperty):
    """
    `Projection <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-projection.html>`__
    """

    props: PropsDictType = {
        "NonKeyAttributes": ([str], False),
        "ProjectionType": (projection_type_validator, False),
    }


class TargetTrackingScalingPolicyConfiguration(AWSProperty):
    """
    `TargetTrackingScalingPolicyConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html>`__
    """

    props: PropsDictType = {
        "DisableScaleIn": (boolean, False),
        "ScaleInCooldown": (integer, False),
        "ScaleOutCooldown": (integer, False),
        "TargetValue": (double, True),
    }


class CapacityAutoScalingSettings(AWSProperty):
    """
    `CapacityAutoScalingSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html>`__
    """

    props: PropsDictType = {
        "MaxCapacity": (integer, True),
        "MinCapacity": (integer, True),
        "SeedCapacity": (integer, False),
        "TargetTrackingScalingPolicyConfiguration": (
            TargetTrackingScalingPolicyConfiguration,
            True,
        ),
    }


class WriteProvisionedThroughputSettings(AWSProperty):
    """
    `WriteProvisionedThroughputSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-writeprovisionedthroughputsettings.html>`__
    """

    props: PropsDictType = {
        "WriteCapacityAutoScalingSettings": (CapacityAutoScalingSettings, False),
    }


class GlobalTableGlobalSecondaryIndex(AWSProperty):
    """
    `GlobalTableGlobalSecondaryIndex <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html>`__
    """

    props: PropsDictType = {
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
        "WriteProvisionedThroughputSettings": (
            WriteProvisionedThroughputSettings,
            False,
        ),
    }


class GlobalTableSSESpecification(AWSProperty):
    """
    `GlobalTableSSESpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-ssespecification.html>`__
    """

    props: PropsDictType = {
        "SSEEnabled": (boolean, True),
        "SSEType": (str, False),
    }


class LocalSecondaryIndex(AWSProperty):
    """
    `LocalSecondaryIndex <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-localsecondaryindex.html>`__
    """

    props: PropsDictType = {
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
    }


class ContributorInsightsSpecification(AWSProperty):
    """
    `ContributorInsightsSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-contributorinsightsspecification.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, True),
    }


class KinesisStreamSpecification(AWSProperty):
    """
    `KinesisStreamSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-kinesisstreamspecification.html>`__
    """

    props: PropsDictType = {
        "StreamArn": (str, True),
    }


class PointInTimeRecoverySpecification(AWSProperty):
    """
    `PointInTimeRecoverySpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-pointintimerecoveryspecification.html>`__
    """

    props: PropsDictType = {
        "PointInTimeRecoveryEnabled": (boolean, False),
    }


class ReadProvisionedThroughputSettings(AWSProperty):
    """
    `ReadProvisionedThroughputSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-readprovisionedthroughputsettings.html>`__
    """

    props: PropsDictType = {
        "ReadCapacityAutoScalingSettings": (CapacityAutoScalingSettings, False),
        "ReadCapacityUnits": (integer, False),
    }


class ReplicaGlobalSecondaryIndexSpecification(AWSProperty):
    """
    `ReplicaGlobalSecondaryIndexSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html>`__
    """

    props: PropsDictType = {
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "IndexName": (str, True),
        "ReadProvisionedThroughputSettings": (ReadProvisionedThroughputSettings, False),
    }


class ReplicaSSESpecification(AWSProperty):
    """
    `ReplicaSSESpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicassespecification.html>`__
    """

    props: PropsDictType = {
        "KMSMasterKeyId": (str, True),
    }


class ReplicaSpecification(AWSProperty):
    """
    `ReplicaSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html>`__
    """

    props: PropsDictType = {
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "DeletionProtectionEnabled": (boolean, False),
        "GlobalSecondaryIndexes": ([ReplicaGlobalSecondaryIndexSpecification], False),
        "KinesisStreamSpecification": (KinesisStreamSpecification, False),
        "PointInTimeRecoverySpecification": (PointInTimeRecoverySpecification, False),
        "ReadProvisionedThroughputSettings": (ReadProvisionedThroughputSettings, False),
        "Region": (str, True),
        "SSESpecification": (ReplicaSSESpecification, False),
        "TableClass": (str, False),
        "Tags": (Tags, False),
    }


class StreamSpecification(AWSProperty):
    """
    `StreamSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-streamspecification.html>`__
    """

    props: PropsDictType = {
        "StreamViewType": (str, True),
    }


class TimeToLiveSpecification(AWSProperty):
    """
    `TimeToLiveSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-timetolivespecification.html>`__
    """

    props: PropsDictType = {
        "AttributeName": (str, False),
        "Enabled": (boolean, True),
    }


class GlobalTable(AWSObject):
    """
    `GlobalTable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html>`__
    """

    resource_type = "AWS::DynamoDB::GlobalTable"

    props: PropsDictType = {
        "AttributeDefinitions": ([AttributeDefinition], True),
        "BillingMode": (str, False),
        "GlobalSecondaryIndexes": ([GlobalTableGlobalSecondaryIndex], False),
        "KeySchema": ([KeySchema], True),
        "LocalSecondaryIndexes": ([LocalSecondaryIndex], False),
        "Replicas": ([ReplicaSpecification], True),
        "SSESpecification": (GlobalTableSSESpecification, False),
        "StreamSpecification": (StreamSpecification, False),
        "TableName": (str, False),
        "TimeToLiveSpecification": (TimeToLiveSpecification, False),
        "WriteProvisionedThroughputSettings": (
            WriteProvisionedThroughputSettings,
            False,
        ),
    }


class ProvisionedThroughput(AWSProperty):
    """
    `ProvisionedThroughput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-provisionedthroughput.html>`__
    """

    props: PropsDictType = {
        "ReadCapacityUnits": (integer, True),
        "WriteCapacityUnits": (integer, True),
    }


class GlobalSecondaryIndex(AWSProperty):
    """
    `GlobalSecondaryIndex <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-globalsecondaryindex.html>`__
    """

    props: PropsDictType = {
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
        "ProvisionedThroughput": (ProvisionedThroughput, False),
    }


class Csv(AWSProperty):
    """
    `Csv <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-csv.html>`__
    """

    props: PropsDictType = {
        "Delimiter": (str, False),
        "HeaderList": ([str], False),
    }


class InputFormatOptions(AWSProperty):
    """
    `InputFormatOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-inputformatoptions.html>`__
    """

    props: PropsDictType = {
        "Csv": (Csv, False),
    }


class S3BucketSource(AWSProperty):
    """
    `S3BucketSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-s3bucketsource.html>`__
    """

    props: PropsDictType = {
        "S3Bucket": (str, True),
        "S3BucketOwner": (str, False),
        "S3KeyPrefix": (str, False),
    }


class ImportSourceSpecification(AWSProperty):
    """
    `ImportSourceSpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-importsourcespecification.html>`__
    """

    props: PropsDictType = {
        "InputCompressionType": (str, False),
        "InputFormat": (str, True),
        "InputFormatOptions": (InputFormatOptions, False),
        "S3BucketSource": (S3BucketSource, True),
    }


class SSESpecification(AWSProperty):
    """
    `SSESpecification <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html>`__
    """

    props: PropsDictType = {
        "KMSMasterKeyId": (str, False),
        "SSEEnabled": (boolean, True),
        "SSEType": (str, False),
    }


class Table(AWSObject):
    """
    `Table <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html>`__
    """

    resource_type = "AWS::DynamoDB::Table"

    props: PropsDictType = {
        "AttributeDefinitions": ([AttributeDefinition], False),
        "BillingMode": (billing_mode_validator, False),
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "DeletionProtectionEnabled": (boolean, False),
        "GlobalSecondaryIndexes": ([GlobalSecondaryIndex], False),
        "ImportSourceSpecification": (ImportSourceSpecification, False),
        "KeySchema": ([KeySchema], True),
        "KinesisStreamSpecification": (KinesisStreamSpecification, False),
        "LocalSecondaryIndexes": ([LocalSecondaryIndex], False),
        "PointInTimeRecoverySpecification": (PointInTimeRecoverySpecification, False),
        "ProvisionedThroughput": (ProvisionedThroughput, False),
        "SSESpecification": (SSESpecification, False),
        "StreamSpecification": (StreamSpecification, False),
        "TableClass": (table_class_validator, False),
        "TableName": (str, False),
        "Tags": (Tags, False),
        "TimeToLiveSpecification": (TimeToLiveSpecification, False),
    }

    def validate(self):
        validate_table(self)



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer
from .validators.globalaccelerator import (
    accelerator_ipaddresstype,
    endpointgroup_healthcheckprotocol,
    listener_clientaffinity,
    listener_protocol,
)


class Accelerator(AWSObject):
    """
    `Accelerator <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-globalaccelerator-accelerator.html>`__
    """

    resource_type = "AWS::GlobalAccelerator::Accelerator"

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "IpAddressType": (accelerator_ipaddresstype, False),
        "IpAddresses": ([str], False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class EndpointConfiguration(AWSProperty):
    """
    `EndpointConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-globalaccelerator-endpointgroup-endpointconfiguration.html>`__
    """

    props: PropsDictType = {
        "AttachmentArn": (str, False),
        "ClientIPPreservationEnabled": (boolean, False),
        "EndpointId": (str, True),
        "Weight": (integer, False),
    }


class PortOverride(AWSProperty):
    """
    `PortOverride <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-globalaccelerator-endpointgroup-portoverride.html>`__
    """

    props: PropsDictType = {
        "EndpointPort": (integer, True),
        "ListenerPort": (integer, True),
    }


class EndpointGroup(AWSObject):
    """
    `EndpointGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-globalaccelerator-endpointgroup.html>`__
    """

    resource_type = "AWS::GlobalAccelerator::EndpointGroup"

    props: PropsDictType = {
        "EndpointConfigurations": ([EndpointConfiguration], False),
        "EndpointGroupRegion": (str, True),
        "HealthCheckIntervalSeconds": (integer, False),
        "HealthCheckPath": (str, False),
        "HealthCheckPort": (integer, False),
        "HealthCheckProtocol": (endpointgroup_healthcheckprotocol, False),
        "ListenerArn": (str, True),
        "PortOverrides": ([PortOverride], False),
        "ThresholdCount": (integer, False),
        "TrafficDialPercentage": (double, False),
    }


class PortRange(AWSProperty):
    """
    `PortRange <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-globalaccelerator-listener-portrange.html>`__
    """

    props: PropsDictType = {
        "FromPort": (integer, True),
        "ToPort": (integer, True),
    }


class Listener(AWSObject):
    """
    `Listener <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-globalaccelerator-listener.html>`__
    """

    resource_type = "AWS::GlobalAccelerator::Listener"

    props: PropsDictType = {
        "AcceleratorArn": (str, True),
        "ClientAffinity": (listener_clientaffinity, False),
        "PortRanges": ([PortRange], True),
        "Protocol": (listener_protocol, True),
    }



# Converted from Route53_A.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import FindInMap, GetAtt, Join, Output, Parameter, Ref, Template
from troposphere.ec2 import Instance
from troposphere.route53 import RecordSetType

t = Template()

t.set_description(
    "AWS CloudFormation Sample Template Route53_A: "
    "Sample template showing how to create an Amazon Route 53 A record that "
    "maps to the public IP address of an EC2 instance. It assumes that you "
    "already have a Hosted Zone registered with Amazon Route 53. **WARNING** "
    "This template creates an Amazon EC2 instance. You will be billed for "
    "the AWS resources used if you create a stack from this template."
)

hostedzone = t.add_parameter(
    Parameter(
        "HostedZone",
        Description="The DNS name of an existing Amazon Route 53 hosted zone",
        Type="String",
    )
)

t.add_mapping(
    "RegionMap",
    {
        "us-east-1": {"AMI": "ami-7f418316"},
        "us-west-1": {"AMI": "ami-951945d0"},
        "us-west-2": {"AMI": "ami-16fd7026"},
        "eu-west-1": {"AMI": "ami-24506250"},
        "sa-east-1": {"AMI": "ami-3e3be423"},
        "ap-southeast-1": {"AMI": "ami-74dda626"},
        "ap-northeast-1": {"AMI": "ami-dcfa4edd"},
    },
)

instance = t.add_resource(
    Instance(
        "Ec2Instance",
        ImageId=FindInMap("RegionMap", Ref("AWS::Region"), "AMI"),
        InstanceType="m1.small",
    )
)

myDNSRecord = t.add_resource(
    RecordSetType(
        "myDNSRecord",
        HostedZoneName=Join("", [Ref(hostedzone), "."]),
        Comment="DNS name for my instance.",
        Name=Join(
            "", [Ref(instance), ".", Ref("AWS::Region"), ".", Ref(hostedzone), "."]
        ),
        Type="A",
        TTL="900",
        ResourceRecords=[GetAtt("Ec2Instance", "PublicIp")],
    )
)


t.add_output(Output("DomainName", Value=Ref(myDNSRecord)))

print(t.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_predictivescalingmaxcapacitybehavior(predictivescalingmaxcapacitybehavior):
    """
    Validate PredictiveScalingMaxCapacityBehavior for ScalingInstruction
    Property: ScalingInstruction.PredictiveScalingMaxCapacityBehavior
    """

    VALID_PREDICTIVESCALINGMAXCAPACITYBEHAVIOR = (
        "SetForecastCapacityToMaxCapacity",
        "SetMaxCapacityToForecastCapacity",
        "SetMaxCapacityAboveForecastCapacity",
    )

    if (
        predictivescalingmaxcapacitybehavior
        not in VALID_PREDICTIVESCALINGMAXCAPACITYBEHAVIOR
    ):  # noqa
        raise ValueError(
            "ScalingInstruction PredictiveScalingMaxCapacityBehavior must be one of: %s"
            % ", ".join(VALID_PREDICTIVESCALINGMAXCAPACITYBEHAVIOR)  # noqa
        )
    return predictivescalingmaxcapacitybehavior


def validate_predictivescalingmode(predictivescalingmode):
    """
    Validate PredictiveScalingMode for ScalingInstruction
    Property: ScalingInstruction.PredictiveScalingMode
    """

    VALID_PREDICTIVESCALINGMODE = ("ForecastAndScale", "ForecastOnly")

    if predictivescalingmode not in VALID_PREDICTIVESCALINGMODE:
        raise ValueError(
            "ScalingInstruction PredictiveScalingMode must be one of: %s"
            % ", ".join(VALID_PREDICTIVESCALINGMODE)  # noqa
        )
    return predictivescalingmode


def validate_scalingpolicyupdatebehavior(scalingpolicyupdatebehavior):
    """
    Validate ScalingPolicyUpdateBehavior for ScalingInstruction
    Property: ScalingInstruction.ScalingPolicyUpdateBehavior
    """

    VALID_SCALINGPOLICYUPDATEBEHAVIOR = (
        "KeepExternalPolicies",
        "ReplaceExternalPolicies",
    )

    if scalingpolicyupdatebehavior not in VALID_SCALINGPOLICYUPDATEBEHAVIOR:
        raise ValueError(
            "ScalingInstruction ScalingPolicyUpdateBehavior must be one of: %s"
            % ", ".join(VALID_SCALINGPOLICYUPDATEBEHAVIOR)  # noqa
        )
    return scalingpolicyupdatebehavior


def scalable_dimension_type(scalable_dimension):
    """
    Property: ScalingInstruction.ScalableDimension
    """
    valid_values = [
        "autoscaling:autoScalingGroup:DesiredCapacity",
        "ecs:service:DesiredCount",
        "ec2:spot-fleet-request:TargetCapacity",
        "rds:cluster:ReadReplicaCount",
        "dynamodb:table:ReadCapacityUnits",
        "dynamodb:table:WriteCapacityUnits",
        "dynamodb:index:ReadCapacityUnits",
        "dynamodb:index:WriteCapacityUnits",
    ]
    if scalable_dimension not in valid_values:
        raise ValueError(
            'ScalableDimension must be one of: "%s"' % (", ".join(valid_values))
        )
    return scalable_dimension


def service_namespace_type(service_namespace):
    """
    Property: ScalingInstruction.ServiceNamespace
    """
    valid_values = ["autoscaling", "ecs", "ec2", "rds", "dynamodb"]
    if service_namespace not in valid_values:
        raise ValueError(
            'ServiceNamespace must be one of: "%s"' % (", ".join(valid_values))
        )
    return service_namespace


def statistic_type(statistic):
    """
    Property: CustomizedScalingMetricSpecification.Statistic
    """
    valid_values = ["Average", "Minimum", "Maximum", "SampleCount", "Sum"]
    if statistic not in valid_values:
        raise ValueError('Statistic must be one of: "%s"' % (", ".join(valid_values)))
    return statistic
